# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Dora(KaitaiStruct):
    """:field openlst_header_uart_start_seq: uart_start_seq
    :field openlst_header_packet_size: packet_size
    :field openlst_header_destination_hwid: destination_hwid
    :field openlst_header_sequence_number: sequence_number
    :field openlst_header_system_byte: system_byte
    :field callsign: callsign
    :field ccsds_version: ccsds_frame.ccsds_header.ccsds_version
    :field ccsds_packet_type: ccsds_frame.ccsds_header.packet_type
    :field ccsds_secondary_header_flag: ccsds_frame.ccsds_header.secondary_header_flag
    :field ccsds_apid: ccsds_frame.ccsds_header.apid
    :field ccsds_sequence_flags: ccsds_frame.ccsds_header.sequence_flags
    :field ccsds_sequence_count: ccsds_frame.ccsds_header.sequence_count
    :field ccsds_packet_length: ccsds_frame.ccsds_header.packet_length
    :field obc_timestamp: ccsds_frame.ccsds_data.timestamp
    :field obc_missionticks: ccsds_frame.ccsds_data.missionticks
    :field obc_state: ccsds_frame.ccsds_data.state
    :field obc_received_packets: ccsds_frame.ccsds_data.received_packets
    :field obc_last_logged_error_thread: ccsds_frame.ccsds_data.last_logged_error_thread
    :field obc_last_logged_error_code: ccsds_frame.ccsds_data.last_logged_error_code
    :field obc_obc_reboot_count: ccsds_frame.ccsds_data.obc_reboot_count
    :field adcs_adcs_commissioning_done_flag: ccsds_frame.ccsds_data.adcs_telemetry.adcs_commissioning_done_flag
    :field adcs_adcs_current_commissioning_state: ccsds_frame.ccsds_data.adcs_telemetry.adcs_current_commissioning_state
    :field adcs_boot_status: ccsds_frame.ccsds_data.adcs_telemetry.boot_status
    :field adcs_boot_count: ccsds_frame.ccsds_data.adcs_telemetry.boot_count
    :field adcs_current_adcs_controlmode: ccsds_frame.ccsds_data.adcs_telemetry.current_adcs_controlmode
    :field adcs_current_adcs_attitudeestimation: ccsds_frame.ccsds_data.adcs_telemetry.current_adcs_attitudeestimation
    :field adcs_cubesense2_enabled: ccsds_frame.ccsds_data.adcs_telemetry.cubesense2_enabled
    :field adcs_cubesense1_enabled: ccsds_frame.ccsds_data.adcs_telemetry.cubesense1_enabled
    :field adcs_cubecontrolmotor_enabled: ccsds_frame.ccsds_data.adcs_telemetry.cubecontrolmotor_enabled
    :field adcs_cubecontrolsignal_enabled: ccsds_frame.ccsds_data.adcs_telemetry.cubecontrolsignal_enabled
    :field adcs_asgp4_mode: ccsds_frame.ccsds_data.adcs_telemetry.asgp4_mode
    :field adcs_currentadcs_adcs_run_mode: ccsds_frame.ccsds_data.adcs_telemetry.currentadcs_adcs_run_mode
    :field adcs_sun_is_above_local_horizon: ccsds_frame.ccsds_data.adcs_telemetry.sun_is_above_local_horizon
    :field adcs_motor_driver_enabled: ccsds_frame.ccsds_data.adcs_telemetry.motor_driver_enabled
    :field adcs_gps_lna_power_enabled: ccsds_frame.ccsds_data.adcs_telemetry.gps_lna_power_enabled
    :field adcs_gps_receiver_enable: ccsds_frame.ccsds_data.adcs_telemetry.gps_receiver_enable
    :field adcs_cubestar_enabled: ccsds_frame.ccsds_data.adcs_telemetry.cubestar_enabled
    :field adcs_cubewheel3_enabled: ccsds_frame.ccsds_data.adcs_telemetry.cubewheel3_enabled
    :field adcs_cubewheel2_enabled: ccsds_frame.ccsds_data.adcs_telemetry.cubewheel2_enabled
    :field adcs_cubewheel1_enabled: ccsds_frame.ccsds_data.adcs_telemetry.cubewheel1_enabled
    :field adcs_cubestar_comms_error: ccsds_frame.ccsds_data.adcs_telemetry.cubestar_comms_error
    :field adcs_cubewheel3_comms_error: ccsds_frame.ccsds_data.adcs_telemetry.cubewheel3_comms_error
    :field adcs_cubewheel2_comms_error: ccsds_frame.ccsds_data.adcs_telemetry.cubewheel2_comms_error
    :field adcs_cubewheel1_comms_error: ccsds_frame.ccsds_data.adcs_telemetry.cubewheel1_comms_error
    :field adcs_cubecontrol_motor_comms_error: ccsds_frame.ccsds_data.adcs_telemetry.cubecontrol_motor_comms_error
    :field adcs_cubecontrol_signal_comms_error: ccsds_frame.ccsds_data.adcs_telemetry.cubecontrol_signal_comms_error
    :field adcs_cubesense2_comms_error: ccsds_frame.ccsds_data.adcs_telemetry.cubesense2_comms_error
    :field adcs_cubesense1_comms_error: ccsds_frame.ccsds_data.adcs_telemetry.cubesense1_comms_error
    :field adcs_cam2_3v3_overcurrent_detected: ccsds_frame.ccsds_data.adcs_telemetry.cam2_3v3_overcurrent_detected
    :field adcs_cam2_sram_overcurrent_detected: ccsds_frame.ccsds_data.adcs_telemetry.cam2_sram_overcurrent_detected
    :field adcs_sun_sensor_range_error: ccsds_frame.ccsds_data.adcs_telemetry.sun_sensor_range_error
    :field adcs_cam1_sensor_detection_error: ccsds_frame.ccsds_data.adcs_telemetry.cam1_sensor_detection_error
    :field adcs_cam1_sensor_busy_error: ccsds_frame.ccsds_data.adcs_telemetry.cam1_sensor_busy_error
    :field adcs_cam1_3v3_overcurrent_detected: ccsds_frame.ccsds_data.adcs_telemetry.cam1_3v3_overcurrent_detected
    :field adcs_cam1_sram_overcurrent_detected: ccsds_frame.ccsds_data.adcs_telemetry.cam1_sram_overcurrent_detected
    :field adcs_magnetometer_range_error: ccsds_frame.ccsds_data.adcs_telemetry.magnetometer_range_error
    :field adcs_star_tracker_overcurrent_detected: ccsds_frame.ccsds_data.adcs_telemetry.star_tracker_overcurrent_detected
    :field adcs_startracker_match_error: ccsds_frame.ccsds_data.adcs_telemetry.startracker_match_error
    :field adcs_coarse_sun_sensor_error: ccsds_frame.ccsds_data.adcs_telemetry.coarse_sun_sensor_error
    :field adcs_wheel_speed_range_error: ccsds_frame.ccsds_data.adcs_telemetry.wheel_speed_range_error
    :field adcs_rate_sensor_range_error: ccsds_frame.ccsds_data.adcs_telemetry.rate_sensor_range_error
    :field adcs_nadir_sensor_range_error: ccsds_frame.ccsds_data.adcs_telemetry.nadir_sensor_range_error
    :field adcs_cam2_sensor_detection_error: ccsds_frame.ccsds_data.adcs_telemetry.cam2_sensor_detection_error
    :field adcs_cam2_sensor_busy_error: ccsds_frame.ccsds_data.adcs_telemetry.cam2_sensor_busy_error
    :field adcs_cubecontrl_cur_3v3: ccsds_frame.ccsds_data.adcs_telemetry.cubecontrl_cur_3v3
    :field adcs_cubecontrl_cur_5v: ccsds_frame.ccsds_data.adcs_telemetry.cubecontrl_cur_5v
    :field adcs_cubecontrol_cur_vbat: ccsds_frame.ccsds_data.adcs_telemetry.cubecontrol_cur_vbat
    :field adcs_fine_est_ang_rates_x: ccsds_frame.ccsds_data.adcs_telemetry.fine_est_ang_rates_x
    :field adcs_fine_est_ang_rates_y: ccsds_frame.ccsds_data.adcs_telemetry.fine_est_ang_rates_y
    :field adcs_fine_est_ang_rates_z: ccsds_frame.ccsds_data.adcs_telemetry.fine_est_ang_rates_z
    :field adcs_fine_sv_x: ccsds_frame.ccsds_data.adcs_telemetry.fine_sv_x
    :field adcs_fine_sv_y: ccsds_frame.ccsds_data.adcs_telemetry.fine_sv_y
    :field adcs_fine_sv_z: ccsds_frame.ccsds_data.adcs_telemetry.fine_sv_z
    :field adcs_mcu_temp: ccsds_frame.ccsds_data.adcs_telemetry.mcu_temp
    :field adcs_temp_mag: ccsds_frame.ccsds_data.adcs_telemetry.temp_mag
    :field adcs_temp_redmag: ccsds_frame.ccsds_data.adcs_telemetry.temp_redmag
    :field adcs_wheel1_current: ccsds_frame.ccsds_data.adcs_telemetry.wheel1_current
    :field adcs_wheel2_current: ccsds_frame.ccsds_data.adcs_telemetry.wheel2_current
    :field adcs_wheel3_current: ccsds_frame.ccsds_data.adcs_telemetry.wheel3_current
    :field adcs_magnetorquer_current: ccsds_frame.ccsds_data.adcs_telemetry.magnetorquer_current
    :field adcs_cubesenes1_3v3_current: ccsds_frame.ccsds_data.adcs_telemetry.cubesenes1_3v3_current
    :field adcs_cubesenes1_sram_current: ccsds_frame.ccsds_data.adcs_telemetry.cubesenes1_sram_current
    :field adcs_cubesenes2_3v3_current: ccsds_frame.ccsds_data.adcs_telemetry.cubesenes2_3v3_current
    :field adcs_cubesenes2_sram_current: ccsds_frame.ccsds_data.adcs_telemetry.cubesenes2_sram_current
    :field eps_output_voltage_battery: ccsds_frame.ccsds_data.eps_telemetry.output_voltage_battery
    :field eps_output_current_battery: ccsds_frame.ccsds_data.eps_telemetry.output_current_battery
    :field eps_bcr_output_current: ccsds_frame.ccsds_data.eps_telemetry.bcr_output_current
    :field eps_motherboard_temperature: ccsds_frame.ccsds_data.eps_telemetry.motherboard_temperature
    :field eps_actual_pdm_states_unused: ccsds_frame.ccsds_data.eps_telemetry.actual_pdm_states_unused
    :field eps_sdr_actual_switch_state: ccsds_frame.ccsds_data.eps_telemetry.sdr_actual_switch_state
    :field eps_cm4_pdm_actual_switch_state: ccsds_frame.ccsds_data.eps_telemetry.cm4_pdm_actual_switch_state
    :field eps_sipm_pdm_actual_switch_state: ccsds_frame.ccsds_data.eps_telemetry.sipm_pdm_actual_switch_state
    :field eps_pdm_actual_switch_state_unused2: ccsds_frame.ccsds_data.eps_telemetry.pdm_actual_switch_state_unused2
    :field eps_daughterboard_temperature: ccsds_frame.ccsds_data.eps_telemetry.daughterboard_temperature
    :field eps_output_current_5v: ccsds_frame.ccsds_data.eps_telemetry.output_current_5v
    :field eps_output_voltage_5v: ccsds_frame.ccsds_data.eps_telemetry.output_voltage_5v
    :field eps_output_current_3v3: ccsds_frame.ccsds_data.eps_telemetry.output_current_3v3
    :field eps_output_voltage_3v3: ccsds_frame.ccsds_data.eps_telemetry.output_voltage_3v3
    :field eps_output_voltage_switch_5: ccsds_frame.ccsds_data.eps_telemetry.output_voltage_switch_5
    :field eps_output_current_switch_5: ccsds_frame.ccsds_data.eps_telemetry.output_current_switch_5
    :field eps_output_voltage_switch_6: ccsds_frame.ccsds_data.eps_telemetry.output_voltage_switch_6
    :field eps_output_current_switch_6: ccsds_frame.ccsds_data.eps_telemetry.output_current_switch_6
    :field eps_output_voltage_switch_7: ccsds_frame.ccsds_data.eps_telemetry.output_voltage_switch_7
    :field eps_output_current_switch_7: ccsds_frame.ccsds_data.eps_telemetry.output_current_switch_7
    :field eps_voltage_feeding_bcr6: ccsds_frame.ccsds_data.eps_telemetry.voltage_feeding_bcr6
    :field eps_current_bcr6_connector_sa6a: ccsds_frame.ccsds_data.eps_telemetry.current_bcr6_connector_sa6a
    :field eps_current_bcr6_connector_sa6b: ccsds_frame.ccsds_data.eps_telemetry.current_bcr6_connector_sa6b
    :field eps_voltage_feeding_bcr7: ccsds_frame.ccsds_data.eps_telemetry.voltage_feeding_bcr7
    :field eps_current_bcr7_connector_sa7a: ccsds_frame.ccsds_data.eps_telemetry.current_bcr7_connector_sa7a
    :field eps_current_bcr7_connector_sa7b: ccsds_frame.ccsds_data.eps_telemetry.current_bcr7_connector_sa7b
    :field eps_voltage_feeding_bcr8: ccsds_frame.ccsds_data.eps_telemetry.voltage_feeding_bcr8
    :field eps_current_bcr8_connector_sa8a: ccsds_frame.ccsds_data.eps_telemetry.current_bcr8_connector_sa8a
    :field eps_current_bcr8_connector_sa8b: ccsds_frame.ccsds_data.eps_telemetry.current_bcr8_connector_sa8b
    :field eps_voltage_feeding_bcr9: ccsds_frame.ccsds_data.eps_telemetry.voltage_feeding_bcr9
    :field eps_current_bcr9_connector_sa9a: ccsds_frame.ccsds_data.eps_telemetry.current_bcr9_connector_sa9a
    :field eps_current_bcr9_connector_sa9b: ccsds_frame.ccsds_data.eps_telemetry.current_bcr9_connector_sa9b
    :field battery_motherboard_temperature: ccsds_frame.ccsds_data.battery_telemetry.motherboard_temperature
    :field battery_output_voltage_battery: ccsds_frame.ccsds_data.battery_telemetry.output_voltage_battery
    :field battery_current_magnitude: ccsds_frame.ccsds_data.battery_telemetry.current_magnitude
    :field battery_daughter_temp1: ccsds_frame.ccsds_data.battery_telemetry.daughter_temp1
    :field battery_daughter_temp2: ccsds_frame.ccsds_data.battery_telemetry.daughter_temp2
    :field battery_daughter_temp3: ccsds_frame.ccsds_data.battery_telemetry.daughter_temp3
    :field battery_heater_status1: ccsds_frame.ccsds_data.battery_telemetry.heater_status1
    :field battery_heater_status2: ccsds_frame.ccsds_data.battery_telemetry.heater_status2
    :field battery_heater_status3: ccsds_frame.ccsds_data.battery_telemetry.heater_status3
    :field battery_current_direction: ccsds_frame.ccsds_data.battery_telemetry.current_direction
    :field payload_sipm_adc_value: ccsds_frame.ccsds_data.payload_telemetry_1.sipm_adc_value
    :field payload_current_payload_schedule: ccsds_frame.ccsds_data.payload_telemetry_1.current_payload_schedule
    :field payload_timestamp: ccsds_frame.ccsds_data.payload_telemetry_1.timestamp
    :field payload_mode: ccsds_frame.ccsds_data.payload_telemetry_1.mode
    :field payload_available_storage: ccsds_frame.ccsds_data.payload_telemetry_1.available_storage
    :field payload_available_memory: ccsds_frame.ccsds_data.payload_telemetry_1.available_memory
    :field payload_cpu_load: ccsds_frame.ccsds_data.payload_telemetry_1.cpu_load
    :field payload_rpi_uptime: ccsds_frame.ccsds_data.payload_telemetry_1.rpi_uptime
    :field payload_reboot_count: ccsds_frame.ccsds_data.payload_telemetry_1.reboot_count
    :field payload_number_of_received_commands: ccsds_frame.ccsds_data.payload_telemetry_1.number_of_received_commands
    :field payload_last_received_command: ccsds_frame.ccsds_data.payload_telemetry_1.last_received_command
    :field payload_error_count: ccsds_frame.ccsds_data.payload_telemetry_1.error_count
    :field payload_filter_bank_power_ch1: ccsds_frame.ccsds_data.payload_telemetry_1.filter_bank_power_ch1
    :field payload_filter_bank_power_ch2: ccsds_frame.ccsds_data.payload_telemetry_1.filter_bank_power_ch2
    :field payload_filter_bank_power_ch2: ccsds_frame.ccsds_data.payload_telemetry_2.filter_bank_power_ch3
    :field payload_filter_bank_power_ch4: ccsds_frame.ccsds_data.payload_telemetry_2.filter_bank_power_ch4
    :field payload_filter_bank_power_ch5: ccsds_frame.ccsds_data.payload_telemetry_2.filter_bank_power_ch5
    :field payload_filter_bank_power_ch6: ccsds_frame.ccsds_data.payload_telemetry_2.filter_bank_power_ch6
    :field payload_filter_bank_power_ch7: ccsds_frame.ccsds_data.payload_telemetry_2.filter_bank_power_ch7
    :field payload_filter_bank_power_ch8: ccsds_frame.ccsds_data.payload_telemetry_2.filter_bank_power_ch8
    :field payload_sdr_power: ccsds_frame.ccsds_data.payload_telemetry_2.sdr_power
    :field payload_sdr_bright_1: ccsds_frame.ccsds_data.payload_telemetry_2.sdr_bright_1
    :field payload_sdr_bright_2: ccsds_frame.ccsds_data.payload_telemetry_2.sdr_bright_2
    :field payload_sdr_bright_3: ccsds_frame.ccsds_data.payload_telemetry_2.sdr_bright_3
    :field payload_sdr_bright_4: ccsds_frame.ccsds_data.payload_telemetry_2.sdr_bright_4
    :field payload_sdr_bright_5: ccsds_frame.ccsds_data.payload_telemetry_2.sdr_bright_5
    :field payload_sdr_bright_6: ccsds_frame.ccsds_data.payload_telemetry_2.sdr_bright_6
    :field payload_sdr_bright_7: ccsds_frame.ccsds_data.payload_telemetry_2.sdr_bright_7
    :field payload_sdr_bright_8: ccsds_frame.ccsds_data.payload_telemetry_2.sdr_bright_8
    :field payload_sdr_bright_9: ccsds_frame.ccsds_data.payload_telemetry_2.sdr_bright_9
    :field payload_sdr_bright_10: ccsds_frame.ccsds_data.payload_telemetry_2.sdr_bright_10
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.uart_start_seq = self._io.read_u2be()
        self.packet_size = self._io.read_u1()
        self.destination_hwid = self._io.read_u2be()
        self.sequence_number = self._io.read_u2be()
        self.system_byte = self._io.read_u1()
        self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
        if not  ((self.callsign == u"KE7DHQ")) :
            raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/seq/5")
        self.dora_syncbytes = self._io.read_bytes(4)
        if not self.dora_syncbytes == b"\x35\x2E\xF8\x53":
            raise kaitaistruct.ValidationNotEqualError(b"\x35\x2E\xF8\x53", self.dora_syncbytes, self._io, u"/seq/6")
        self.ccsds_frame = Dora.CcsdsFrame(self._io, self, self._root)

    class BatteryTelemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.motherboard_temperature_raw = self._io.read_u2be()
            self.output_voltage_battery_raw = self._io.read_u2be()
            self.current_magnitude_raw = self._io.read_u2be()
            self.daughter_temp1_raw = self._io.read_u2be()
            self.daughter_temp2_raw = self._io.read_u2be()
            self.daughter_temp3_raw = self._io.read_u2be()
            self.heater_status1 = self._io.read_bits_int_be(1) != 0
            self.heater_status2 = self._io.read_bits_int_be(1) != 0
            self.heater_status3 = self._io.read_bits_int_be(1) != 0
            self.current_direction = self._io.read_bits_int_be(5)

        @property
        def output_voltage_battery(self):
            if hasattr(self, '_m_output_voltage_battery'):
                return self._m_output_voltage_battery

            self._m_output_voltage_battery = (self.output_voltage_battery_raw * 0.008993)
            return getattr(self, '_m_output_voltage_battery', None)

        @property
        def daughter_temp1(self):
            if hasattr(self, '_m_daughter_temp1'):
                return self._m_daughter_temp1

            self._m_daughter_temp1 = ((self.daughter_temp1_raw * 0.397600) - 238.57)
            return getattr(self, '_m_daughter_temp1', None)

        @property
        def current_magnitude(self):
            if hasattr(self, '_m_current_magnitude'):
                return self._m_current_magnitude

            self._m_current_magnitude = (self.current_magnitude_raw * 14.662757)
            return getattr(self, '_m_current_magnitude', None)

        @property
        def daughter_temp3(self):
            if hasattr(self, '_m_daughter_temp3'):
                return self._m_daughter_temp3

            self._m_daughter_temp3 = ((self.daughter_temp3_raw * 0.397600) - 238.57)
            return getattr(self, '_m_daughter_temp3', None)

        @property
        def daughter_temp2(self):
            if hasattr(self, '_m_daughter_temp2'):
                return self._m_daughter_temp2

            self._m_daughter_temp2 = ((self.daughter_temp2_raw * 0.397600) - 238.57)
            return getattr(self, '_m_daughter_temp2', None)

        @property
        def motherboard_temperature(self):
            if hasattr(self, '_m_motherboard_temperature'):
                return self._m_motherboard_temperature

            self._m_motherboard_temperature = ((self.motherboard_temperature_raw * 0.372434) - 273.15)
            return getattr(self, '_m_motherboard_temperature', None)


    class CcsdsHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ccsds_version = self._io.read_bits_int_be(3)
            self.packet_type = self._io.read_bits_int_be(1) != 0
            self.secondary_header_flag = self._io.read_bits_int_be(1) != 0
            self.apid = self._io.read_bits_int_be(11)
            if not  ((self.apid == 3)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.apid, self._io, u"/types/ccsds_header/seq/3")
            self.sequence_flags = self._io.read_bits_int_be(2)
            self.sequence_count = self._io.read_bits_int_be(14)
            self.packet_length = self._io.read_bits_int_be(16)


    class PayloadTelemetry2(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.filter_bank_power_ch3 = self._io.read_u1()
            self.filter_bank_power_ch4 = self._io.read_u1()
            self.filter_bank_power_ch5 = self._io.read_u1()
            self.filter_bank_power_ch6 = self._io.read_u1()
            self.filter_bank_power_ch7 = self._io.read_u1()
            self.filter_bank_power_ch8 = self._io.read_u1()
            self.sdr_power = self._io.read_u2be()
            self.sdr_bright_1 = self._io.read_u2be()
            self.sdr_bright_2 = self._io.read_u2be()
            self.sdr_bright_3 = self._io.read_u2be()
            self.sdr_bright_4 = self._io.read_u2be()
            self.sdr_bright_5 = self._io.read_u2be()
            self.sdr_bright_6 = self._io.read_u2be()
            self.sdr_bright_7 = self._io.read_u2be()
            self.sdr_bright_8 = self._io.read_u2be()
            self.sdr_bright_9 = self._io.read_u2be()
            self.sdr_bright_10 = self._io.read_u2be()


    class PayloadTelemetry1(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sipm_adc_value = self._io.read_u2be()
            self.current_payload_schedule = self._io.read_u2be()
            self.timestamp_raw = self._io.read_u8be()
            self.mode = self._io.read_u1()
            self.available_storage = self._io.read_u4be()
            self.available_memory = self._io.read_u4be()
            self.cpu_load = self._io.read_u4be()
            self.rpi_uptime = self._io.read_u4be()
            self.reboot_count = self._io.read_u4be()
            self.number_of_received_commands = self._io.read_u4be()
            self.last_received_command = self._io.read_u1()
            self.error_count = self._io.read_u4be()
            self.filter_bank_power_ch1 = self._io.read_u1()
            self.filter_bank_power_ch2 = self._io.read_u1()

        @property
        def timestamp(self):
            if hasattr(self, '_m_timestamp'):
                return self._m_timestamp

            self._m_timestamp = (self.timestamp_raw & 4294967295)
            return getattr(self, '_m_timestamp', None)


    class AdcsTelemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.adcs_commissioning_done_flag = self._io.read_bits_int_be(1) != 0
            self.adcs_current_commissioning_state = self._io.read_bits_int_be(7)
            self._io.align_to_byte()
            self.boot_status = self._io.read_u1()
            self.boot_count = self._io.read_u2be()
            self.current_adcs_controlmode = self._io.read_bits_int_be(4)
            self.current_adcs_attitudeestimation = self._io.read_bits_int_be(4)
            self.cubesense2_enabled = self._io.read_bits_int_be(1) != 0
            self.cubesense1_enabled = self._io.read_bits_int_be(1) != 0
            self.cubecontrolmotor_enabled = self._io.read_bits_int_be(1) != 0
            self.cubecontrolsignal_enabled = self._io.read_bits_int_be(1) != 0
            self.asgp4_mode = self._io.read_bits_int_be(2)
            self.currentadcs_adcs_run_mode = self._io.read_bits_int_be(2)
            self.sun_is_above_local_horizon = self._io.read_bits_int_be(1) != 0
            self.motor_driver_enabled = self._io.read_bits_int_be(1) != 0
            self.gps_lna_power_enabled = self._io.read_bits_int_be(1) != 0
            self.gps_receiver_enable = self._io.read_bits_int_be(1) != 0
            self.cubestar_enabled = self._io.read_bits_int_be(1) != 0
            self.cubewheel3_enabled = self._io.read_bits_int_be(1) != 0
            self.cubewheel2_enabled = self._io.read_bits_int_be(1) != 0
            self.cubewheel1_enabled = self._io.read_bits_int_be(1) != 0
            self.cubestar_comms_error = self._io.read_bits_int_be(1) != 0
            self.cubewheel3_comms_error = self._io.read_bits_int_be(1) != 0
            self.cubewheel2_comms_error = self._io.read_bits_int_be(1) != 0
            self.cubewheel1_comms_error = self._io.read_bits_int_be(1) != 0
            self.cubecontrol_motor_comms_error = self._io.read_bits_int_be(1) != 0
            self.cubecontrol_signal_comms_error = self._io.read_bits_int_be(1) != 0
            self.cubesense2_comms_error = self._io.read_bits_int_be(1) != 0
            self.cubesense1_comms_error = self._io.read_bits_int_be(1) != 0
            self.cam2_3v3_overcurrent_detected = self._io.read_bits_int_be(1) != 0
            self.cam2_sram_overcurrent_detected = self._io.read_bits_int_be(1) != 0
            self.sun_sensor_range_error = self._io.read_bits_int_be(1) != 0
            self.cam1_sensor_detection_error = self._io.read_bits_int_be(1) != 0
            self.cam1_sensor_busy_error = self._io.read_bits_int_be(1) != 0
            self.cam1_3v3_overcurrent_detected = self._io.read_bits_int_be(1) != 0
            self.cam1_sram_overcurrent_detected = self._io.read_bits_int_be(1) != 0
            self.magnetometer_range_error = self._io.read_bits_int_be(1) != 0
            self.star_tracker_overcurrent_detected = self._io.read_bits_int_be(1) != 0
            self.startracker_match_error = self._io.read_bits_int_be(1) != 0
            self.coarse_sun_sensor_error = self._io.read_bits_int_be(1) != 0
            self.wheel_speed_range_error = self._io.read_bits_int_be(1) != 0
            self.rate_sensor_range_error = self._io.read_bits_int_be(1) != 0
            self.nadir_sensor_range_error = self._io.read_bits_int_be(1) != 0
            self.cam2_sensor_detection_error = self._io.read_bits_int_be(1) != 0
            self.cam2_sensor_busy_error = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.cubecontrl_cur_3v3_raw = self._io.read_u2le()
            self.cubecontrl_cur_5v_raw = self._io.read_u2le()
            self.cubecontrol_cur_vbat_raw = self._io.read_u2le()
            self.fine_est_ang_rates_x_raw = self._io.read_u2le()
            self.fine_est_ang_rates_y_raw = self._io.read_u2le()
            self.fine_est_ang_rates_z_raw = self._io.read_u2le()
            self.fine_sv_x_raw = self._io.read_u2le()
            self.fine_sv_y_raw = self._io.read_u2le()
            self.fine_sv_z_raw = self._io.read_u2le()
            self.mcu_temp = self._io.read_u2le()
            self.temp_mag_raw = self._io.read_u2le()
            self.temp_redmag_raw = self._io.read_u2le()
            self.wheel1_current_raw = self._io.read_u2le()
            self.wheel2_current_raw = self._io.read_u2le()
            self.wheel3_current_raw = self._io.read_u2le()
            self.magnetorquer_current_raw = self._io.read_u2le()
            self.cubesenes1_3v3_current_raw = self._io.read_u2le()
            self.cubesenes1_sram_current_raw = self._io.read_u2le()
            self.cubesenes2_3v3_current_raw = self._io.read_u2le()
            self.cubesenes2_sram_current_raw = self._io.read_u2le()

        @property
        def fine_est_ang_rates_x(self):
            if hasattr(self, '_m_fine_est_ang_rates_x'):
                return self._m_fine_est_ang_rates_x

            self._m_fine_est_ang_rates_x = (self.fine_est_ang_rates_x_raw * 0.001)
            return getattr(self, '_m_fine_est_ang_rates_x', None)

        @property
        def temp_redmag(self):
            if hasattr(self, '_m_temp_redmag'):
                return self._m_temp_redmag

            self._m_temp_redmag = (self.temp_redmag_raw * 0.1)
            return getattr(self, '_m_temp_redmag', None)

        @property
        def temp_mag(self):
            if hasattr(self, '_m_temp_mag'):
                return self._m_temp_mag

            self._m_temp_mag = (self.temp_mag_raw * 0.1)
            return getattr(self, '_m_temp_mag', None)

        @property
        def cubecontrl_cur_5v(self):
            if hasattr(self, '_m_cubecontrl_cur_5v'):
                return self._m_cubecontrl_cur_5v

            self._m_cubecontrl_cur_5v = (self.cubecontrl_cur_5v_raw * 0.48828125)
            return getattr(self, '_m_cubecontrl_cur_5v', None)

        @property
        def wheel3_current(self):
            if hasattr(self, '_m_wheel3_current'):
                return self._m_wheel3_current

            self._m_wheel3_current = (self.wheel3_current_raw * 0.01)
            return getattr(self, '_m_wheel3_current', None)

        @property
        def cubesenes2_3v3_current(self):
            if hasattr(self, '_m_cubesenes2_3v3_current'):
                return self._m_cubesenes2_3v3_current

            self._m_cubesenes2_3v3_current = (self.cubesenes2_3v3_current_raw * 0.01)
            return getattr(self, '_m_cubesenes2_3v3_current', None)

        @property
        def cubesenes1_3v3_current(self):
            if hasattr(self, '_m_cubesenes1_3v3_current'):
                return self._m_cubesenes1_3v3_current

            self._m_cubesenes1_3v3_current = (self.cubesenes1_3v3_current_raw * 0.01)
            return getattr(self, '_m_cubesenes1_3v3_current', None)

        @property
        def wheel2_current(self):
            if hasattr(self, '_m_wheel2_current'):
                return self._m_wheel2_current

            self._m_wheel2_current = (self.wheel2_current_raw * 0.01)
            return getattr(self, '_m_wheel2_current', None)

        @property
        def fine_est_ang_rates_z(self):
            if hasattr(self, '_m_fine_est_ang_rates_z'):
                return self._m_fine_est_ang_rates_z

            self._m_fine_est_ang_rates_z = (self.fine_est_ang_rates_z_raw * 0.001)
            return getattr(self, '_m_fine_est_ang_rates_z', None)

        @property
        def fine_sv_y(self):
            if hasattr(self, '_m_fine_sv_y'):
                return self._m_fine_sv_y

            self._m_fine_sv_y = (self.fine_sv_y_raw * 0.0001)
            return getattr(self, '_m_fine_sv_y', None)

        @property
        def fine_est_ang_rates_y(self):
            if hasattr(self, '_m_fine_est_ang_rates_y'):
                return self._m_fine_est_ang_rates_y

            self._m_fine_est_ang_rates_y = (self.fine_est_ang_rates_y_raw * 0.001)
            return getattr(self, '_m_fine_est_ang_rates_y', None)

        @property
        def magnetorquer_current(self):
            if hasattr(self, '_m_magnetorquer_current'):
                return self._m_magnetorquer_current

            self._m_magnetorquer_current = (self.magnetorquer_current_raw * 0.01)
            return getattr(self, '_m_magnetorquer_current', None)

        @property
        def cubecontrol_cur_vbat(self):
            if hasattr(self, '_m_cubecontrol_cur_vbat'):
                return self._m_cubecontrol_cur_vbat

            self._m_cubecontrol_cur_vbat = (self.cubecontrol_cur_vbat_raw * 0.48828125)
            return getattr(self, '_m_cubecontrol_cur_vbat', None)

        @property
        def cubecontrl_cur_3v3(self):
            if hasattr(self, '_m_cubecontrl_cur_3v3'):
                return self._m_cubecontrl_cur_3v3

            self._m_cubecontrl_cur_3v3 = (self.cubecontrl_cur_3v3_raw * 0.48828125)
            return getattr(self, '_m_cubecontrl_cur_3v3', None)

        @property
        def wheel1_current(self):
            if hasattr(self, '_m_wheel1_current'):
                return self._m_wheel1_current

            self._m_wheel1_current = (self.wheel1_current_raw * 0.01)
            return getattr(self, '_m_wheel1_current', None)

        @property
        def cubesenes1_sram_current(self):
            if hasattr(self, '_m_cubesenes1_sram_current'):
                return self._m_cubesenes1_sram_current

            self._m_cubesenes1_sram_current = (self.cubesenes1_sram_current_raw * 0.01)
            return getattr(self, '_m_cubesenes1_sram_current', None)

        @property
        def fine_sv_x(self):
            if hasattr(self, '_m_fine_sv_x'):
                return self._m_fine_sv_x

            self._m_fine_sv_x = (self.fine_sv_x_raw * 0.0001)
            return getattr(self, '_m_fine_sv_x', None)

        @property
        def cubesenes2_sram_current(self):
            if hasattr(self, '_m_cubesenes2_sram_current'):
                return self._m_cubesenes2_sram_current

            self._m_cubesenes2_sram_current = (self.cubesenes2_sram_current_raw * 0.01)
            return getattr(self, '_m_cubesenes2_sram_current', None)

        @property
        def fine_sv_z(self):
            if hasattr(self, '_m_fine_sv_z'):
                return self._m_fine_sv_z

            self._m_fine_sv_z = (self.fine_sv_z_raw * 0.0001)
            return getattr(self, '_m_fine_sv_z', None)


    class FirstDoraPacket(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.timestamp = self._io.read_u8be()
            self.missionticks_raw = self._io.read_u8be()
            self.state = self._io.read_u1()
            self.received_packets = self._io.read_u4be()
            self.last_logged_error_thread = self._io.read_u1()
            self.last_logged_error_code = self._io.read_u1()
            self.obc_reboot_count = self._io.read_u4be()
            self.adcs_telemetry = Dora.AdcsTelemetry(self._io, self, self._root)
            self.eps_telemetry = Dora.EpsTelemetry(self._io, self, self._root)
            self.battery_telemetry = Dora.BatteryTelemetry(self._io, self, self._root)
            self.payload_telemetry_1 = Dora.PayloadTelemetry1(self._io, self, self._root)

        @property
        def missionticks(self):
            if hasattr(self, '_m_missionticks'):
                return self._m_missionticks

            self._m_missionticks = (self.missionticks_raw & 281474976710655)
            return getattr(self, '_m_missionticks', None)


    class EpsTelemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.output_voltage_battery_raw = self._io.read_u2be()
            self.output_current_battery_raw = self._io.read_u2be()
            self.bcr_output_current_raw = self._io.read_u2be()
            self.motherboard_temperature_raw = self._io.read_u2be()
            self.actual_pdm_states_unused = self._io.read_bits_int_be(8)
            self.sdr_actual_switch_state = self._io.read_bits_int_be(1) != 0
            self.cm4_pdm_actual_switch_state = self._io.read_bits_int_be(1) != 0
            self.sipm_pdm_actual_switch_state = self._io.read_bits_int_be(1) != 0
            self.pdm_actual_switch_state_unused2 = self._io.read_bits_int_be(5)
            self._io.align_to_byte()
            self.daughterboard_temperature_raw = self._io.read_u2be()
            self.output_current_5v_raw = self._io.read_u2be()
            self.output_voltage_5v_raw = self._io.read_u2be()
            self.output_current_3v3_raw = self._io.read_u2be()
            self.output_voltage_3v3_raw = self._io.read_u2be()
            self.output_voltage_switch_5_raw = self._io.read_u2be()
            self.output_current_switch_5_raw = self._io.read_u2be()
            self.output_voltage_switch_6_raw = self._io.read_u2be()
            self.output_current_switch_6_raw = self._io.read_u2be()
            self.output_voltage_switch_7_raw = self._io.read_u2be()
            self.output_current_switch_7_raw = self._io.read_u2be()
            self.voltage_feeding_bcr6_raw = self._io.read_u2be()
            self.current_bcr6_connector_sa6a_raw = self._io.read_u2be()
            self.current_bcr6_connector_sa6b_raw = self._io.read_u2be()
            self.voltage_feeding_bcr7_raw = self._io.read_u2be()
            self.current_bcr7_connector_sa7a_raw = self._io.read_u2be()
            self.current_bcr7_connector_sa7b_raw = self._io.read_u2be()
            self.voltage_feeding_bcr8_raw = self._io.read_u2be()
            self.current_bcr8_connector_sa8a_raw = self._io.read_u2be()
            self.current_bcr8_connector_sa8b_raw = self._io.read_u2be()
            self.voltage_feeding_bcr9_raw = self._io.read_u2be()
            self.current_bcr9_connector_sa9a_raw = self._io.read_u2be()
            self.current_bcr9_connector_sa9b_raw = self._io.read_u2be()

        @property
        def bcr_output_current(self):
            if hasattr(self, '_m_bcr_output_current'):
                return self._m_bcr_output_current

            self._m_bcr_output_current = (self.bcr_output_current_raw * 0.014662757)
            return getattr(self, '_m_bcr_output_current', None)

        @property
        def daughterboard_temperature(self):
            if hasattr(self, '_m_daughterboard_temperature'):
                return self._m_daughterboard_temperature

            self._m_daughterboard_temperature = ((self.motherboard_temperature_raw * 0.372434) - 273.15)
            return getattr(self, '_m_daughterboard_temperature', None)

        @property
        def output_current_switch_6(self):
            if hasattr(self, '_m_output_current_switch_6'):
                return self._m_output_current_switch_6

            self._m_output_current_switch_6 = (self.output_current_switch_6_raw * 0.001328)
            return getattr(self, '_m_output_current_switch_6', None)

        @property
        def output_voltage_battery(self):
            if hasattr(self, '_m_output_voltage_battery'):
                return self._m_output_voltage_battery

            self._m_output_voltage_battery = (self.output_voltage_battery_raw * 0.008978)
            return getattr(self, '_m_output_voltage_battery', None)

        @property
        def output_current_switch_7(self):
            if hasattr(self, '_m_output_current_switch_7'):
                return self._m_output_current_switch_7

            self._m_output_current_switch_7 = (self.output_current_switch_7_raw * 0.001328)
            return getattr(self, '_m_output_current_switch_7', None)

        @property
        def voltage_feeding_bcr9(self):
            if hasattr(self, '_m_voltage_feeding_bcr9'):
                return self._m_voltage_feeding_bcr9

            self._m_voltage_feeding_bcr9 = (self.voltage_feeding_bcr9_raw * 0.0322581)
            return getattr(self, '_m_voltage_feeding_bcr9', None)

        @property
        def current_bcr7_connector_sa7a(self):
            if hasattr(self, '_m_current_bcr7_connector_sa7a'):
                return self._m_current_bcr7_connector_sa7a

            self._m_current_bcr7_connector_sa7a = (self.current_bcr7_connector_sa7a_raw * 0.0009775)
            return getattr(self, '_m_current_bcr7_connector_sa7a', None)

        @property
        def current_bcr7_connector_sa7b(self):
            if hasattr(self, '_m_current_bcr7_connector_sa7b'):
                return self._m_current_bcr7_connector_sa7b

            self._m_current_bcr7_connector_sa7b = (self.current_bcr7_connector_sa7b_raw * 0.0009775)
            return getattr(self, '_m_current_bcr7_connector_sa7b', None)

        @property
        def output_current_battery(self):
            if hasattr(self, '_m_output_current_battery'):
                return self._m_output_current_battery

            self._m_output_current_battery = (self.output_current_battery_raw * 0.00681988679)
            return getattr(self, '_m_output_current_battery', None)

        @property
        def current_bcr9_connector_sa9b(self):
            if hasattr(self, '_m_current_bcr9_connector_sa9b'):
                return self._m_current_bcr9_connector_sa9b

            self._m_current_bcr9_connector_sa9b = (self.current_bcr9_connector_sa9b_raw * 0.0009775)
            return getattr(self, '_m_current_bcr9_connector_sa9b', None)

        @property
        def output_current_5v(self):
            if hasattr(self, '_m_output_current_5v'):
                return self._m_output_current_5v

            self._m_output_current_5v = (self.output_current_5v_raw * 0.00681988679)
            return getattr(self, '_m_output_current_5v', None)

        @property
        def output_voltage_5v(self):
            if hasattr(self, '_m_output_voltage_5v'):
                return self._m_output_voltage_5v

            self._m_output_voltage_5v = (self.output_voltage_5v_raw * 0.005865)
            return getattr(self, '_m_output_voltage_5v', None)

        @property
        def output_voltage_3v3(self):
            if hasattr(self, '_m_output_voltage_3v3'):
                return self._m_output_voltage_3v3

            self._m_output_voltage_3v3 = (self.output_voltage_3v3_raw * 0.004311)
            return getattr(self, '_m_output_voltage_3v3', None)

        @property
        def voltage_feeding_bcr8(self):
            if hasattr(self, '_m_voltage_feeding_bcr8'):
                return self._m_voltage_feeding_bcr8

            self._m_voltage_feeding_bcr8 = (self.voltage_feeding_bcr8_raw * 0.0322581)
            return getattr(self, '_m_voltage_feeding_bcr8', None)

        @property
        def voltage_feeding_bcr6(self):
            if hasattr(self, '_m_voltage_feeding_bcr6'):
                return self._m_voltage_feeding_bcr6

            self._m_voltage_feeding_bcr6 = (self.voltage_feeding_bcr6_raw * 0.0322581)
            return getattr(self, '_m_voltage_feeding_bcr6', None)

        @property
        def current_bcr9_connector_sa9a(self):
            if hasattr(self, '_m_current_bcr9_connector_sa9a'):
                return self._m_current_bcr9_connector_sa9a

            self._m_current_bcr9_connector_sa9a = (self.current_bcr9_connector_sa9a_raw * 0.0009775)
            return getattr(self, '_m_current_bcr9_connector_sa9a', None)

        @property
        def output_current_switch_5(self):
            if hasattr(self, '_m_output_current_switch_5'):
                return self._m_output_current_switch_5

            self._m_output_current_switch_5 = (self.output_current_switch_5_raw * 0.001328)
            return getattr(self, '_m_output_current_switch_5', None)

        @property
        def output_current_3v3(self):
            if hasattr(self, '_m_output_current_3v3'):
                return self._m_output_current_3v3

            self._m_output_current_3v3 = (self.output_current_3v3_raw * 0.00681988679)
            return getattr(self, '_m_output_current_3v3', None)

        @property
        def output_voltage_switch_6(self):
            if hasattr(self, '_m_output_voltage_switch_6'):
                return self._m_output_voltage_switch_6

            self._m_output_voltage_switch_6 = (self.output_voltage_switch_6_raw * 0.005865)
            return getattr(self, '_m_output_voltage_switch_6', None)

        @property
        def motherboard_temperature(self):
            if hasattr(self, '_m_motherboard_temperature'):
                return self._m_motherboard_temperature

            self._m_motherboard_temperature = ((self.motherboard_temperature_raw * 0.372434) - 273.15)
            return getattr(self, '_m_motherboard_temperature', None)

        @property
        def output_voltage_switch_5(self):
            if hasattr(self, '_m_output_voltage_switch_5'):
                return self._m_output_voltage_switch_5

            self._m_output_voltage_switch_5 = (self.output_voltage_switch_5_raw * 0.005865)
            return getattr(self, '_m_output_voltage_switch_5', None)

        @property
        def output_voltage_switch_7(self):
            if hasattr(self, '_m_output_voltage_switch_7'):
                return self._m_output_voltage_switch_7

            self._m_output_voltage_switch_7 = (self.output_voltage_switch_7_raw * 0.005865)
            return getattr(self, '_m_output_voltage_switch_7', None)

        @property
        def current_bcr8_connector_sa8a(self):
            if hasattr(self, '_m_current_bcr8_connector_sa8a'):
                return self._m_current_bcr8_connector_sa8a

            self._m_current_bcr8_connector_sa8a = (self.current_bcr8_connector_sa8a_raw * 0.0009775)
            return getattr(self, '_m_current_bcr8_connector_sa8a', None)

        @property
        def current_bcr8_connector_sa8b(self):
            if hasattr(self, '_m_current_bcr8_connector_sa8b'):
                return self._m_current_bcr8_connector_sa8b

            self._m_current_bcr8_connector_sa8b = (self.current_bcr8_connector_sa8b_raw * 0.0009775)
            return getattr(self, '_m_current_bcr8_connector_sa8b', None)

        @property
        def current_bcr6_connector_sa6b(self):
            if hasattr(self, '_m_current_bcr6_connector_sa6b'):
                return self._m_current_bcr6_connector_sa6b

            self._m_current_bcr6_connector_sa6b = (self.current_bcr6_connector_sa6b_raw * 0.0009775)
            return getattr(self, '_m_current_bcr6_connector_sa6b', None)

        @property
        def voltage_feeding_bcr7(self):
            if hasattr(self, '_m_voltage_feeding_bcr7'):
                return self._m_voltage_feeding_bcr7

            self._m_voltage_feeding_bcr7 = (self.voltage_feeding_bcr7_raw * 0.0322581)
            return getattr(self, '_m_voltage_feeding_bcr7', None)

        @property
        def current_bcr6_connector_sa6a(self):
            if hasattr(self, '_m_current_bcr6_connector_sa6a'):
                return self._m_current_bcr6_connector_sa6a

            self._m_current_bcr6_connector_sa6a = (self.current_bcr6_connector_sa6a_raw * 0.0009775)
            return getattr(self, '_m_current_bcr6_connector_sa6a', None)


    class SecondDoraPacket(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.payload_telemetry_2 = Dora.PayloadTelemetry2(self._io, self, self._root)


    class CcsdsFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ccsds_header = Dora.CcsdsHeader(self._io, self, self._root)
            _on = self.ccsds_header.sequence_flags
            if _on == 1:
                self.ccsds_data = Dora.FirstDoraPacket(self._io, self, self._root)
            elif _on == 2:
                self.ccsds_data = Dora.SecondDoraPacket(self._io, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
from enum import Enum


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Irvine(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field spacecraft_response: ax25_frame.payload.ax25_info.body.body.spacecraft_response
    :field spacecraft_id: ax25_frame.payload.ax25_info.body.body.spacecraft_id
    :field ldc: ax25_frame.payload.ax25_info.body.body.ldc
    :field gyro: ax25_frame.payload.ax25_info.body.body.gyro
    :field mag: ax25_frame.payload.ax25_info.body.body.mag
    :field daughter_a_tmp_sensor: ax25_frame.payload.ax25_info.body.body.daughter_a_tmp_sensor
    :field three_v_pl_tmp_sensor: ax25_frame.payload.ax25_info.body.body.three_v_pl_tmp_sensor
    :field temp_nz: ax25_frame.payload.ax25_info.body.body.temp_nz
    :field volt3v: ax25_frame.payload.ax25_info.body.body.volt3v
    :field curr3v: ax25_frame.payload.ax25_info.body.body.curr3v
    :field volt5vpl: ax25_frame.payload.ax25_info.body.body.volt5vpl
    :field curr5vpl: ax25_frame.payload.ax25_info.body.body.curr5vpl
    :field src_port: ax25_frame.payload.ax25_info.body.body.src_port
    :field dst_port: ax25_frame.payload.ax25_info.body.body.dst_port
    :field src_ip_addr: ax25_frame.payload.ax25_info.src_ip_addr
    :field dst_ip_addr: ax25_frame.payload.ax25_info.dst_ip_addr
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Irvine.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Irvine.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Irvine.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Irvine.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Irvine.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Irvine.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Irvine.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Irvine.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Irvine.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Irvine.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Irvine.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Irvine.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            _on = self.pid
            if _on == 204:
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Irvine.Ipv4Pkt(_io__raw_ax25_info, self, self._root)
            elif _on == 240:
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Irvine.NoneL3(_io__raw_ax25_info, self, self._root)
            else:
                self.ax25_info = self._io.read_bytes_full()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class UdpPayload(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.spacecraft_response = self._io.read_u1()
            self.spacecraft_id = (self._io.read_bytes_term(0, False, True, True)).decode(u"ASCII")
            self.ldc = self._io.read_u2be()
            self.gyro = []
            for i in range(3):
                self.gyro.append(self._io.read_s4be())

            self.mag = []
            for i in range(3):
                self.mag.append(self._io.read_s4be())

            self.daughter_a_tmp_sensor = self._io.read_s2be()
            self.three_v_pl_tmp_sensor = self._io.read_s2be()
            self.temp_nz = self._io.read_s2be()
            self.volt3v = self._io.read_s4be()
            self.curr3v = self._io.read_s4be()
            self.volt5vpl = self._io.read_s4be()
            self.curr5vpl = self._io.read_s4be()

        @property
        def gyro_y(self):
            if hasattr(self, '_m_gyro_y'):
                return self._m_gyro_y

            self._m_gyro_y = (self.gyro[1] / (1024.0 * 1024.0))
            return getattr(self, '_m_gyro_y', None)

        @property
        def three_v_pl_tmp_sensor_val_k(self):
            if hasattr(self, '_m_three_v_pl_tmp_sensor_val_k'):
                return self._m_three_v_pl_tmp_sensor_val_k

            self._m_three_v_pl_tmp_sensor_val_k = (self.three_v_pl_tmp_sensor / 64.0)
            return getattr(self, '_m_three_v_pl_tmp_sensor_val_k', None)

        @property
        def curr3v_val(self):
            if hasattr(self, '_m_curr3v_val'):
                return self._m_curr3v_val

            self._m_curr3v_val = (self.curr3v / (256.0 * 256.0))
            return getattr(self, '_m_curr3v_val', None)

        @property
        def ldc_hrs(self):
            if hasattr(self, '_m_ldc_hrs'):
                return self._m_ldc_hrs

            self._m_ldc_hrs = (self.ldc_mins / 60.0)
            return getattr(self, '_m_ldc_hrs', None)

        @property
        def mag_z(self):
            """Magnitudes in [nT]!."""
            if hasattr(self, '_m_mag_z'):
                return self._m_mag_z

            self._m_mag_z = (self.mag[2] / (1024.0 * 1024.0))
            return getattr(self, '_m_mag_z', None)

        @property
        def gyro_x(self):
            if hasattr(self, '_m_gyro_x'):
                return self._m_gyro_x

            self._m_gyro_x = (self.gyro[0] / (1024.0 * 1024.0))
            return getattr(self, '_m_gyro_x', None)

        @property
        def ldc_secs(self):
            if hasattr(self, '_m_ldc_secs'):
                return self._m_ldc_secs

            self._m_ldc_secs = (self.ldc * 256.0)
            return getattr(self, '_m_ldc_secs', None)

        @property
        def gyro_z(self):
            """Gyroscope in [deg/s]!."""
            if hasattr(self, '_m_gyro_z'):
                return self._m_gyro_z

            self._m_gyro_z = (self.gyro[2] / (1024.0 * 1024.0))
            return getattr(self, '_m_gyro_z', None)

        @property
        def curr5vpl_val(self):
            """Voltages in [V]! Currents in [A]!
            IRVINE payload inside a UDP datagram multicast packet
            Currently partially reverse-engineered
            Conversion values taken from source-code
            """
            if hasattr(self, '_m_curr5vpl_val'):
                return self._m_curr5vpl_val

            self._m_curr5vpl_val = (self.curr5vpl / (256.0 * 256.0))
            return getattr(self, '_m_curr5vpl_val', None)

        @property
        def three_v_pl_tmp_sensor_val_deg_c(self):
            if hasattr(self, '_m_three_v_pl_tmp_sensor_val_deg_c'):
                return self._m_three_v_pl_tmp_sensor_val_deg_c

            self._m_three_v_pl_tmp_sensor_val_deg_c = (self.three_v_pl_tmp_sensor_val_k - 273.15)
            return getattr(self, '_m_three_v_pl_tmp_sensor_val_deg_c', None)

        @property
        def mag_x(self):
            if hasattr(self, '_m_mag_x'):
                return self._m_mag_x

            self._m_mag_x = (self.mag[0] / (1024.0 * 1024.0))
            return getattr(self, '_m_mag_x', None)

        @property
        def volt5vpl_val(self):
            if hasattr(self, '_m_volt5vpl_val'):
                return self._m_volt5vpl_val

            self._m_volt5vpl_val = (self.volt5vpl / (256.0 * 256.0))
            return getattr(self, '_m_volt5vpl_val', None)

        @property
        def temp_nz_val_k(self):
            """Temperatures in [K]!."""
            if hasattr(self, '_m_temp_nz_val_k'):
                return self._m_temp_nz_val_k

            self._m_temp_nz_val_k = (self.temp_nz / 64.0)
            return getattr(self, '_m_temp_nz_val_k', None)

        @property
        def daughter_a_tmp_sensor_val_k(self):
            if hasattr(self, '_m_daughter_a_tmp_sensor_val_k'):
                return self._m_daughter_a_tmp_sensor_val_k

            self._m_daughter_a_tmp_sensor_val_k = (self.daughter_a_tmp_sensor / 64.0)
            return getattr(self, '_m_daughter_a_tmp_sensor_val_k', None)

        @property
        def daughter_a_tmp_sensor_val_deg_c(self):
            if hasattr(self, '_m_daughter_a_tmp_sensor_val_deg_c'):
                return self._m_daughter_a_tmp_sensor_val_deg_c

            self._m_daughter_a_tmp_sensor_val_deg_c = (self.daughter_a_tmp_sensor_val_k - 273.15)
            return getattr(self, '_m_daughter_a_tmp_sensor_val_deg_c', None)

        @property
        def mag_y(self):
            if hasattr(self, '_m_mag_y'):
                return self._m_mag_y

            self._m_mag_y = (self.mag[1] / (1024.0 * 1024.0))
            return getattr(self, '_m_mag_y', None)

        @property
        def ldc_mins(self):
            if hasattr(self, '_m_ldc_mins'):
                return self._m_ldc_mins

            self._m_ldc_mins = (self.ldc_secs / 60.0)
            return getattr(self, '_m_ldc_mins', None)

        @property
        def volt3v_val(self):
            if hasattr(self, '_m_volt3v_val'):
                return self._m_volt3v_val

            self._m_volt3v_val = (self.volt3v / (256.0 * 256.0))
            return getattr(self, '_m_volt3v_val', None)

        @property
        def temp_nz_val_deg_c(self):
            """Temperatures in [C]!."""
            if hasattr(self, '_m_temp_nz_val_deg_c'):
                return self._m_temp_nz_val_deg_c

            self._m_temp_nz_val_deg_c = (self.temp_nz_val_k - 273.15)
            return getattr(self, '_m_temp_nz_val_deg_c', None)


    class Ipv4Options(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.entries = []
            i = 0
            while not self._io.is_eof():
                self.entries.append(Irvine.Ipv4Option(self._io, self, self._root))
                i += 1



    class NoNextHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            pass


    class Ipv6Pkt(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.version = self._io.read_bits_int_be(4)
            self.traffic_class = self._io.read_bits_int_be(8)
            self.flow_label = self._io.read_bits_int_be(20)
            self._io.align_to_byte()
            self.payload_length = self._io.read_u2be()
            self.next_header_type = self._io.read_u1()
            self.hop_limit = self._io.read_u1()
            self.src_ipv6_addr = self._io.read_bytes(16)
            self.dst_ipv6_addr = self._io.read_bytes(16)
            _on = self.next_header_type
            if _on == 17:
                self.next_header = Irvine.UdpDtgrm(self._io, self, self._root)
            elif _on == 0:
                self.next_header = Irvine.OptionHopByHop(self._io, self, self._root)
            elif _on == 4:
                self.next_header = Irvine.Ipv4Pkt(self._io, self, self._root)
            elif _on == 6:
                self.next_header = Irvine.TcpSegm(self._io, self, self._root)
            elif _on == 59:
                self.next_header = Irvine.NoNextHeader(self._io, self, self._root)
            self.rest = self._io.read_bytes_full()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class Ipv4Pkt(KaitaiStruct):

        class ProtocolEnum(Enum):
            hopopt = 0
            icmp = 1
            igmp = 2
            ggp = 3
            ipv4 = 4
            st = 5
            tcp = 6
            cbt = 7
            egp = 8
            igp = 9
            bbn_rcc_mon = 10
            nvp_ii = 11
            pup = 12
            argus = 13
            emcon = 14
            xnet = 15
            chaos = 16
            udp = 17
            mux = 18
            dcn_meas = 19
            hmp = 20
            prm = 21
            xns_idp = 22
            trunk_1 = 23
            trunk_2 = 24
            leaf_1 = 25
            leaf_2 = 26
            rdp = 27
            irtp = 28
            iso_tp4 = 29
            netblt = 30
            mfe_nsp = 31
            merit_inp = 32
            dccp = 33
            x_3pc = 34
            idpr = 35
            xtp = 36
            ddp = 37
            idpr_cmtp = 38
            tp_plus_plus = 39
            il = 40
            ipv6 = 41
            sdrp = 42
            ipv6_route = 43
            ipv6_frag = 44
            idrp = 45
            rsvp = 46
            gre = 47
            dsr = 48
            bna = 49
            esp = 50
            ah = 51
            i_nlsp = 52
            swipe = 53
            narp = 54
            mobile = 55
            tlsp = 56
            skip = 57
            ipv6_icmp = 58
            ipv6_nonxt = 59
            ipv6_opts = 60
            any_host_internal_protocol = 61
            cftp = 62
            any_local_network = 63
            sat_expak = 64
            kryptolan = 65
            rvd = 66
            ippc = 67
            any_distributed_file_system = 68
            sat_mon = 69
            visa = 70
            ipcv = 71
            cpnx = 72
            cphb = 73
            wsn = 74
            pvp = 75
            br_sat_mon = 76
            sun_nd = 77
            wb_mon = 78
            wb_expak = 79
            iso_ip = 80
            vmtp = 81
            secure_vmtp = 82
            vines = 83
            ttp = 84
            nsfnet_igp = 85
            dgp = 86
            tcf = 87
            eigrp = 88
            ospfigp = 89
            sprite_rpc = 90
            larp = 91
            mtp = 92
            ax_25 = 93
            ipip = 94
            micp = 95
            scc_sp = 96
            etherip = 97
            encap = 98
            any_private_encryption_scheme = 99
            gmtp = 100
            ifmp = 101
            pnni = 102
            pim = 103
            aris = 104
            scps = 105
            qnx = 106
            a_n = 107
            ipcomp = 108
            snp = 109
            compaq_peer = 110
            ipx_in_ip = 111
            vrrp = 112
            pgm = 113
            any_0_hop = 114
            l2tp = 115
            ddx = 116
            iatp = 117
            stp = 118
            srp = 119
            uti = 120
            smp = 121
            sm = 122
            ptp = 123
            isis_over_ipv4 = 124
            fire = 125
            crtp = 126
            crudp = 127
            sscopmce = 128
            iplt = 129
            sps = 130
            pipe = 131
            sctp = 132
            fc = 133
            rsvp_e2e_ignore = 134
            mobility_header = 135
            udplite = 136
            mpls_in_ip = 137
            manet = 138
            hip = 139
            shim6 = 140
            wesp = 141
            rohc = 142
            reserved_255 = 255
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.b1 = self._io.read_u1()
            self.b2 = self._io.read_u1()
            self.total_length = self._io.read_u2be()
            self.identification = self._io.read_u2be()
            self.b67 = self._io.read_u2be()
            self.ttl = self._io.read_u1()
            self.protocol = KaitaiStream.resolve_enum(Irvine.Ipv4Pkt.ProtocolEnum, self._io.read_u1())
            self.header_checksum = self._io.read_u2be()
            self.src_ip_addr = self._io.read_bytes(4)
            self.dst_ip_addr = self._io.read_bytes(4)
            self._raw_options = self._io.read_bytes((self.ihl_bytes - 20))
            _io__raw_options = KaitaiStream(BytesIO(self._raw_options))
            self.options = Irvine.Ipv4Options(_io__raw_options, self, self._root)
            _on = self.protocol
            if _on == Irvine.Ipv4Pkt.ProtocolEnum.udp:
                self._raw_body = self._io.read_bytes((self.total_length - self.ihl_bytes))
                _io__raw_body = KaitaiStream(BytesIO(self._raw_body))
                self.body = Irvine.UdpDtgrm(_io__raw_body, self, self._root)
            elif _on == Irvine.Ipv4Pkt.ProtocolEnum.icmp:
                self._raw_body = self._io.read_bytes((self.total_length - self.ihl_bytes))
                _io__raw_body = KaitaiStream(BytesIO(self._raw_body))
                self.body = Irvine.IcmpPkt(_io__raw_body, self, self._root)
            elif _on == Irvine.Ipv4Pkt.ProtocolEnum.ipv6:
                self._raw_body = self._io.read_bytes((self.total_length - self.ihl_bytes))
                _io__raw_body = KaitaiStream(BytesIO(self._raw_body))
                self.body = Irvine.Ipv6Pkt(_io__raw_body, self, self._root)
            elif _on == Irvine.Ipv4Pkt.ProtocolEnum.tcp:
                self._raw_body = self._io.read_bytes((self.total_length - self.ihl_bytes))
                _io__raw_body = KaitaiStream(BytesIO(self._raw_body))
                self.body = Irvine.TcpSegm(_io__raw_body, self, self._root)
            else:
                self.body = self._io.read_bytes((self.total_length - self.ihl_bytes))

        @property
        def version(self):
            if hasattr(self, '_m_version'):
                return self._m_version

            self._m_version = ((self.b1 & 240) >> 4)
            return getattr(self, '_m_version', None)

        @property
        def ihl(self):
            if hasattr(self, '_m_ihl'):
                return self._m_ihl

            self._m_ihl = (self.b1 & 15)
            return getattr(self, '_m_ihl', None)

        @property
        def ihl_bytes(self):
            if hasattr(self, '_m_ihl_bytes'):
                return self._m_ihl_bytes

            self._m_ihl_bytes = (self.ihl * 4)
            return getattr(self, '_m_ihl_bytes', None)


    class TcpSegm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.src_port = self._io.read_u2be()
            self.dst_port = self._io.read_u2be()
            self.seq_num = self._io.read_u4be()
            self.ack_num = self._io.read_u4be()
            self.b12 = self._io.read_u1()
            self.b13 = self._io.read_u1()
            self.window_size = self._io.read_u2be()
            self.checksum = self._io.read_u2be()
            self.urgent_pointer = self._io.read_u2be()
            self.body = self._io.read_bytes_full()


    class NoneL3(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data = self._io.read_bytes_full()


    class UdpDtgrm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.src_port = self._io.read_u2be()
            self.dst_port = self._io.read_u2be()
            self.length = self._io.read_u2be()
            self.checksum = self._io.read_u2be()
            self._raw_body = self._io.read_bytes_full()
            _io__raw_body = KaitaiStream(BytesIO(self._raw_body))
            self.body = Irvine.UdpPayload(_io__raw_body, self, self._root)


    class DestinationUnreachableMsg(KaitaiStruct):

        class DestinationUnreachableCode(Enum):
            net_unreachable = 0
            host_unreachable = 1
            protocol_unreachable = 2
            port_unreachable = 3
            fragmentation_needed_and_df_set = 4
            source_route_failed = 5
            dst_net_unkown = 6
            sdt_host_unkown = 7
            src_isolated = 8
            net_prohibited_by_admin = 9
            host_prohibited_by_admin = 10
            net_unreachable_for_tos = 11
            host_unreachable_for_tos = 12
            communication_prohibited_by_admin = 13
            host_precedence_violation = 14
            precedence_cuttoff_in_effect = 15
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.code = KaitaiStream.resolve_enum(Irvine.DestinationUnreachableMsg.DestinationUnreachableCode, self._io.read_u1())
            self.checksum = self._io.read_u2be()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Irvine.Callsign(_io__raw_callsign_ror, self, self._root)


    class OptionHopByHop(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.next_header_type = self._io.read_u1()
            self.hdr_ext_len = self._io.read_u1()
            self.body = self._io.read_bytes((self.hdr_ext_len - 1))
            _on = self.next_header_type
            if _on == 0:
                self.next_header = Irvine.OptionHopByHop(self._io, self, self._root)
            elif _on == 6:
                self.next_header = Irvine.TcpSegm(self._io, self, self._root)
            elif _on == 59:
                self.next_header = Irvine.NoNextHeader(self._io, self, self._root)


    class EchoMsg(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.code = self._io.read_bytes(1)
            if not self.code == b"\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x00", self.code, self._io, u"/types/echo_msg/seq/0")
            self.checksum = self._io.read_u2be()
            self.identifier = self._io.read_u2be()
            self.seq_num = self._io.read_u2be()
            self.data = self._io.read_bytes_full()


    class TimeExceededMsg(KaitaiStruct):

        class TimeExceededCode(Enum):
            time_to_live_exceeded_in_transit = 0
            fragment_reassembly_time_exceeded = 1
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.code = KaitaiStream.resolve_enum(Irvine.TimeExceededMsg.TimeExceededCode, self._io.read_u1())
            self.checksum = self._io.read_u2be()


    class IcmpPkt(KaitaiStruct):

        class IcmpTypeEnum(Enum):
            echo_reply = 0
            destination_unreachable = 3
            source_quench = 4
            redirect = 5
            echo = 8
            time_exceeded = 11
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.icmp_type = KaitaiStream.resolve_enum(Irvine.IcmpPkt.IcmpTypeEnum, self._io.read_u1())
            if self.icmp_type == Irvine.IcmpPkt.IcmpTypeEnum.destination_unreachable:
                self.destination_unreachable = Irvine.DestinationUnreachableMsg(self._io, self, self._root)

            if self.icmp_type == Irvine.IcmpPkt.IcmpTypeEnum.time_exceeded:
                self.time_exceeded = Irvine.TimeExceededMsg(self._io, self, self._root)

            if  ((self.icmp_type == Irvine.IcmpPkt.IcmpTypeEnum.echo) or (self.icmp_type == Irvine.IcmpPkt.IcmpTypeEnum.echo_reply)) :
                self.echo = Irvine.EchoMsg(self._io, self, self._root)



    class Ipv4Option(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.b1 = self._io.read_u1()
            self.len = self._io.read_u1()
            self.body = self._io.read_bytes(((self.len - 2) if self.len > 2 else 0))

        @property
        def copy(self):
            if hasattr(self, '_m_copy'):
                return self._m_copy

            self._m_copy = ((self.b1 & 128) >> 7)
            return getattr(self, '_m_copy', None)

        @property
        def opt_class(self):
            if hasattr(self, '_m_opt_class'):
                return self._m_opt_class

            self._m_opt_class = ((self.b1 & 96) >> 5)
            return getattr(self, '_m_opt_class', None)

        @property
        def number(self):
            if hasattr(self, '_m_number'):
                return self._m_number

            self._m_number = (self.b1 & 31)
            return getattr(self, '_m_number', None)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Yomogi(KaitaiStruct):
    """:field beacon_type: yomogi.type_check.beacon_type
    :field header: yomogi.type_check.header
    :field rssi_temp_or_sms: yomogi.type_check.rssi_temp_or_sms
    :field type_id: yomogi.type_check.type_id
    :field batt_v: yomogi.type_check.batt_v
    :field batt_i: yomogi.type_check.batt_i
    :field batt_t: yomogi.type_check.batt_t
    :field ad_counter: yomogi.type_check.ad_counter
    :field batt_heater: yomogi.type_check.batt_heater
    :field kill_sw_main: yomogi.type_check.kill_sw_main
    :field kill_sw_fab: yomogi.type_check.kill_sw_fab
    :field sap_minus_x: yomogi.type_check.sap_minus_x
    :field sap_plus_y: yomogi.type_check.sap_plus_y
    :field sap_minus_y: yomogi.type_check.sap_minus_y
    :field sap_plus_z: yomogi.type_check.sap_plus_z
    :field sap_minus_z: yomogi.type_check.sap_minus_z
    :field time_after_reset: yomogi.type_check.time_after_reset
    :field raw_i: yomogi.type_check.raw_i
    :field reset_raw_v_mon: yomogi.type_check.reset_raw_v_mon
    :field lcl_uhf_com: yomogi.type_check.lcl_uhf_com
    :field lcl_5v0: yomogi.type_check.lcl_5v0
    :field dcdc_fab: yomogi.type_check.dcdc_fab
    :field lcl_depant: yomogi.type_check.lcl_depant
    :field lcl_3v3_2: yomogi.type_check.lcl_3v3_2
    :field lcl_fab: yomogi.type_check.lcl_fab
    :field dcdc_5v0: yomogi.type_check.dcdc_5v0
    :field dcdc_3v3_2: yomogi.type_check.dcdc_3v3_2
    :field lcl_compic: yomogi.type_check.lcl_compic
    :field lcl_mainpic: yomogi.type_check.lcl_mainpic
    :field gmsk_cmd_counter: yomogi.type_check.gmsk_cmd_counter
    :field reserve_cmd_counter: yomogi.type_check.reserve_cmd_counter
    :field uplink_success: yomogi.type_check.uplink_success
    :field kill_counter: yomogi.type_check.kill_counter
    :field bpb_t: yomogi.type_check.bpb_t
    :field cw_beacon: yomogi.type_check.cw_beacon
    :field hk_header_2: yomogi.type_check.hk_header_2
    :field hk_seconds: yomogi.type_check.hk_seconds
    :field hk_minutes: yomogi.type_check.hk_minutes
    :field hk_hours: yomogi.type_check.hk_hours
    :field hk_days: yomogi.type_check.hk_days
    :field hk_temp_plus_x: yomogi.type_check.hk_temp_plus_x
    :field hk_temp_minus_x: yomogi.type_check.hk_temp_minus_x
    :field hk_temp_plus_y: yomogi.type_check.hk_temp_plus_y
    :field hk_temp_minus_y: yomogi.type_check.hk_temp_minus_y
    :field hk_temp_plus_z: yomogi.type_check.hk_temp_plus_z
    :field hk_temp_minus_z: yomogi.type_check.hk_temp_minus_z
    :field hk_bpb_t: yomogi.type_check.hk_bpb_t
    :field hk_voltage_minus_x: yomogi.type_check.hk_voltage_minus_x
    :field hk_voltage_plus_y: yomogi.type_check.hk_voltage_plus_y
    :field hk_voltage_minus_y: yomogi.type_check.hk_voltage_minus_y
    :field hk_voltage_plus_z: yomogi.type_check.hk_voltage_plus_z
    :field hk_voltage_minus_z: yomogi.type_check.hk_voltage_minus_z
    :field hk_current_minus_x: yomogi.type_check.hk_current_minus_x
    :field hk_current_plus_y: yomogi.type_check.hk_current_plus_y
    :field hk_current_minus_y: yomogi.type_check.hk_current_minus_y
    :field hk_current_plus_z: yomogi.type_check.hk_current_plus_z
    :field hk_current_minus_z: yomogi.type_check.hk_current_minus_z
    :field hk_batt_t: yomogi.type_check.hk_batt_t
    :field hk_batt_v: yomogi.type_check.hk_batt_v
    :field hk_batt_i: yomogi.type_check.hk_batt_i
    :field hk_fab_raw_v: yomogi.type_check.hk_fab_raw_v
    :field hk_fab_raw_i: yomogi.type_check.hk_fab_raw_i
    :field hk_src_v: yomogi.type_check.hk_src_v
    :field hk_src_i: yomogi.type_check.hk_src_i
    :field hk_heater_flag: yomogi.type_check.hk_heater_flag
    :field hk_kill_sw: yomogi.type_check.hk_kill_sw
    :field hk_reset_raw_v: yomogi.type_check.hk_reset_raw_v
    :field hk_v3_3_no1_i: yomogi.type_check.hk_v3_3_no1_i
    :field hk_v3_3_no2_i: yomogi.type_check.hk_v3_3_no2_i
    :field hk_uhf_com_i: yomogi.type_check.hk_uhf_com_i
    :field hk_ant_dep_i: yomogi.type_check.hk_ant_dep_i
    :field hk_v5_0_i: yomogi.type_check.hk_v5_0_i
    :field hk_reset_sw_status: yomogi.type_check.hk_reset_sw_status
    :field hk_reserve_cmd_0_cmd0: yomogi.type_check.hk_reserve_cmd_0_cmd0
    :field hk_reserve_cmd_0_cmd1: yomogi.type_check.hk_reserve_cmd_0_cmd1
    :field hk_reserve_cmd_0_cmd2: yomogi.type_check.hk_reserve_cmd_0_cmd2
    :field hk_reserve_cmd_0_cmd3: yomogi.type_check.hk_reserve_cmd_0_cmd3
    :field hk_reserve_cmd_0_cmd4: yomogi.type_check.hk_reserve_cmd_0_cmd4
    :field hk_reserve_cmd_0_cmd5: yomogi.type_check.hk_reserve_cmd_0_cmd5
    :field hk_reserve_cmd_0_cmd6: yomogi.type_check.hk_reserve_cmd_0_cmd6
    :field hk_reserve_cmd_0_cmd7: yomogi.type_check.hk_reserve_cmd_0_cmd7
    :field hk_reserve_cmd_1_cmd0: yomogi.type_check.hk_reserve_cmd_1_cmd0
    :field hk_reserve_cmd_1_cmd1: yomogi.type_check.hk_reserve_cmd_1_cmd1
    :field hk_reserve_cmd_1_cmd2: yomogi.type_check.hk_reserve_cmd_1_cmd2
    :field hk_reserve_cmd_1_cmd3: yomogi.type_check.hk_reserve_cmd_1_cmd3
    :field hk_reserve_cmd_1_cmd4: yomogi.type_check.hk_reserve_cmd_1_cmd4
    :field hk_reserve_cmd_1_cmd5: yomogi.type_check.hk_reserve_cmd_1_cmd5
    :field hk_reserve_cmd_1_cmd6: yomogi.type_check.hk_reserve_cmd_1_cmd6
    :field hk_reserve_cmd_1_cmd7: yomogi.type_check.hk_reserve_cmd_1_cmd7
    :field digi_dest_callsign: yomogi.type_check.ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field digi_src_callsign: yomogi.type_check.ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field digi_src_ssid: yomogi.type_check.ax25_frame.ax25_header.src_ssid_raw.ssid
    :field digi_dest_ssid: yomogi.type_check.ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_instance___callsign: yomogi.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_callsign_raw.callsign_ror.callsign
    :field rpt_instance___ssid: yomogi.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.ssid
    :field rpt_instance___hbit: yomogi.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.hbit
    :field digi_ctl: yomogi.type_check.ax25_frame.ax25_header.ctl
    :field digi_pid: yomogi.type_check.ax25_frame.ax25_header.pid
    :field digi_message: yomogi.type_check.ax25_frame.ax25_info.digi_message
          
    
    .. seealso::
       Source - https://sites.google.com/s.chibakoudai.jp/gardens-01-yomogi/eng/satellite-eng/documents/transmission-format
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.yomogi = Yomogi.YomogiT(self._io, self, self._root)

    class YomogiT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.format_identifier
            if _on == 8750332926247903338:
                self.type_check = Yomogi.Cw(self._io, self, self._root)
            elif _on == 5352306439146123338:
                self.type_check = Yomogi.Hk(self._io, self, self._root)
            else:
                self.type_check = Yomogi.Aprs(self._io, self, self._root)

        @property
        def format_identifier(self):
            if hasattr(self, '_m_format_identifier'):
                return self._m_format_identifier

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_format_identifier = self._io.read_u8be()
            self._io.seek(_pos)
            return getattr(self, '_m_format_identifier', None)


    class Cw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header = (self._io.read_bytes(14)).decode(u"ASCII")
            if not self.header == u"yomogi js1ymx ":
                raise kaitaistruct.ValidationNotEqualError(u"yomogi js1ymx ", self.header, self._io, u"/types/cw/seq/0")
            self.rssi_temp_or_sms = (self._io.read_bytes(6)).decode(u"ASCII")
            self.space = (self._io.read_bytes(1)).decode(u"ASCII")
            if not self.space == u" ":
                raise kaitaistruct.ValidationNotEqualError(u" ", self.space, self._io, u"/types/cw/seq/2")
            self.byte_1 = self._io.read_u1()
            self.byte_2 = self._io.read_u1()
            self.byte_3 = self._io.read_u1()
            self.byte_4 = self._io.read_u1()
            self.byte_5 = self._io.read_u1()
            self.lengthcheck = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def reserve_cmd_counter(self):
            if hasattr(self, '_m_reserve_cmd_counter'):
                return self._m_reserve_cmd_counter

            if self.type_id == 1:
                self._m_reserve_cmd_counter = ((self.byte_4 & 14) >> 1)

            return getattr(self, '_m_reserve_cmd_counter', None)

        @property
        def batt_heater(self):
            if hasattr(self, '_m_batt_heater'):
                return self._m_batt_heater

            if self.type_id == 0:
                self._m_batt_heater = ((self.byte_4 & 8) >> 3)

            return getattr(self, '_m_batt_heater', None)

        @property
        def byte_3_hex_left(self):
            if hasattr(self, '_m_byte_3_hex_left'):
                return self._m_byte_3_hex_left

            self._m_byte_3_hex_left = self.byte_3 // 16
            return getattr(self, '_m_byte_3_hex_left', None)

        @property
        def reset_raw_v_mon(self):
            if hasattr(self, '_m_reset_raw_v_mon'):
                return self._m_reset_raw_v_mon

            if self.type_id == 1:
                self._m_reset_raw_v_mon = (self.byte_2 & 1)

            return getattr(self, '_m_reset_raw_v_mon', None)

        @property
        def lcl_uhf_com(self):
            if hasattr(self, '_m_lcl_uhf_com'):
                return self._m_lcl_uhf_com

            if self.type_id == 1:
                self._m_lcl_uhf_com = ((self.byte_2 & 2) >> 1)

            return getattr(self, '_m_lcl_uhf_com', None)

        @property
        def necessary_for_lengthcheck(self):
            if hasattr(self, '_m_necessary_for_lengthcheck'):
                return self._m_necessary_for_lengthcheck

            if len(self.lengthcheck) != 0:
                self._m_necessary_for_lengthcheck = int(self.lengthcheck) // 0

            return getattr(self, '_m_necessary_for_lengthcheck', None)

        @property
        def lcl_fab(self):
            if hasattr(self, '_m_lcl_fab'):
                return self._m_lcl_fab

            if self.type_id == 1:
                self._m_lcl_fab = ((self.byte_2 & 64) >> 6)

            return getattr(self, '_m_lcl_fab', None)

        @property
        def lcl_5v0(self):
            if hasattr(self, '_m_lcl_5v0'):
                return self._m_lcl_5v0

            if self.type_id == 1:
                self._m_lcl_5v0 = ((self.byte_2 & 4) >> 2)

            return getattr(self, '_m_lcl_5v0', None)

        @property
        def byte_4_hex_right_digit(self):
            if hasattr(self, '_m_byte_4_hex_right_digit'):
                return self._m_byte_4_hex_right_digit

            self._m_byte_4_hex_right_digit = (u"a" if str(self.byte_4_hex_right) == u"10" else (u"b" if str(self.byte_4_hex_right) == u"11" else (u"c" if str(self.byte_4_hex_right) == u"12" else (u"d" if str(self.byte_4_hex_right) == u"13" else (u"e" if str(self.byte_4_hex_right) == u"14" else (u"f" if str(self.byte_4_hex_right) == u"15" else str(self.byte_4_hex_right)))))))
            return getattr(self, '_m_byte_4_hex_right_digit', None)

        @property
        def ad_counter(self):
            if hasattr(self, '_m_ad_counter'):
                return self._m_ad_counter

            if self.type_id == 0:
                self._m_ad_counter = ((self.byte_4 & 6) >> 1)

            return getattr(self, '_m_ad_counter', None)

        @property
        def sap_minus_y(self):
            if hasattr(self, '_m_sap_minus_y'):
                return self._m_sap_minus_y

            if self.type_id == 0:
                self._m_sap_minus_y = (self.byte_5 & 1)

            return getattr(self, '_m_sap_minus_y', None)

        @property
        def byte_3_hex_right_digit(self):
            if hasattr(self, '_m_byte_3_hex_right_digit'):
                return self._m_byte_3_hex_right_digit

            self._m_byte_3_hex_right_digit = (u"a" if str(self.byte_3_hex_right) == u"10" else (u"b" if str(self.byte_3_hex_right) == u"11" else (u"c" if str(self.byte_3_hex_right) == u"12" else (u"d" if str(self.byte_3_hex_right) == u"13" else (u"e" if str(self.byte_3_hex_right) == u"14" else (u"f" if str(self.byte_3_hex_right) == u"15" else str(self.byte_3_hex_right)))))))
            return getattr(self, '_m_byte_3_hex_right_digit', None)

        @property
        def byte_1_hex_right(self):
            if hasattr(self, '_m_byte_1_hex_right'):
                return self._m_byte_1_hex_right

            self._m_byte_1_hex_right = (self.byte_1 % 16)
            return getattr(self, '_m_byte_1_hex_right', None)

        @property
        def batt_t(self):
            if hasattr(self, '_m_batt_t'):
                return self._m_batt_t

            if self.type_id == 0:
                self._m_batt_t = self.byte_3

            return getattr(self, '_m_batt_t', None)

        @property
        def byte_1_hex_left_digit(self):
            if hasattr(self, '_m_byte_1_hex_left_digit'):
                return self._m_byte_1_hex_left_digit

            self._m_byte_1_hex_left_digit = (u"a" if str(self.byte_1_hex_left) == u"10" else (u"b" if str(self.byte_1_hex_left) == u"11" else (u"c" if str(self.byte_1_hex_left) == u"12" else (u"d" if str(self.byte_1_hex_left) == u"13" else (u"e" if str(self.byte_1_hex_left) == u"14" else (u"f" if str(self.byte_1_hex_left) == u"15" else str(self.byte_1_hex_left)))))))
            return getattr(self, '_m_byte_1_hex_left_digit', None)

        @property
        def byte_3_hex(self):
            if hasattr(self, '_m_byte_3_hex'):
                return self._m_byte_3_hex

            self._m_byte_3_hex = self.byte_3_hex_left_digit + self.byte_3_hex_right_digit
            return getattr(self, '_m_byte_3_hex', None)

        @property
        def byte_5_hex_left_digit(self):
            if hasattr(self, '_m_byte_5_hex_left_digit'):
                return self._m_byte_5_hex_left_digit

            self._m_byte_5_hex_left_digit = (u"a" if str(self.byte_5_hex_left) == u"10" else (u"b" if str(self.byte_5_hex_left) == u"11" else (u"c" if str(self.byte_5_hex_left) == u"12" else (u"d" if str(self.byte_5_hex_left) == u"13" else (u"e" if str(self.byte_5_hex_left) == u"14" else (u"f" if str(self.byte_5_hex_left) == u"15" else str(self.byte_5_hex_left)))))))
            return getattr(self, '_m_byte_5_hex_left_digit', None)

        @property
        def byte_5_hex(self):
            if hasattr(self, '_m_byte_5_hex'):
                return self._m_byte_5_hex

            self._m_byte_5_hex = self.byte_5_hex_left_digit + self.byte_5_hex_right_digit
            return getattr(self, '_m_byte_5_hex', None)

        @property
        def byte_2_hex_left(self):
            if hasattr(self, '_m_byte_2_hex_left'):
                return self._m_byte_2_hex_left

            self._m_byte_2_hex_left = self.byte_2 // 16
            return getattr(self, '_m_byte_2_hex_left', None)

        @property
        def bpb_t(self):
            if hasattr(self, '_m_bpb_t'):
                return self._m_bpb_t

            if self.type_id == 1:
                self._m_bpb_t = self.byte_5

            return getattr(self, '_m_bpb_t', None)

        @property
        def lcl_depant(self):
            if hasattr(self, '_m_lcl_depant'):
                return self._m_lcl_depant

            if self.type_id == 1:
                self._m_lcl_depant = ((self.byte_2 & 16) >> 4)

            return getattr(self, '_m_lcl_depant', None)

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = (u"CW" if 0 == 0 else u"CW")
            return getattr(self, '_m_beacon_type', None)

        @property
        def byte_2_hex_right(self):
            if hasattr(self, '_m_byte_2_hex_right'):
                return self._m_byte_2_hex_right

            self._m_byte_2_hex_right = (self.byte_2 % 16)
            return getattr(self, '_m_byte_2_hex_right', None)

        @property
        def byte_1_hex_left(self):
            if hasattr(self, '_m_byte_1_hex_left'):
                return self._m_byte_1_hex_left

            self._m_byte_1_hex_left = self.byte_1 // 16
            return getattr(self, '_m_byte_1_hex_left', None)

        @property
        def byte_5_hex_right(self):
            if hasattr(self, '_m_byte_5_hex_right'):
                return self._m_byte_5_hex_right

            self._m_byte_5_hex_right = (self.byte_5 % 16)
            return getattr(self, '_m_byte_5_hex_right', None)

        @property
        def byte_1_hex_right_digit(self):
            if hasattr(self, '_m_byte_1_hex_right_digit'):
                return self._m_byte_1_hex_right_digit

            self._m_byte_1_hex_right_digit = (u"a" if str(self.byte_1_hex_right) == u"10" else (u"b" if str(self.byte_1_hex_right) == u"11" else (u"c" if str(self.byte_1_hex_right) == u"12" else (u"d" if str(self.byte_1_hex_right) == u"13" else (u"e" if str(self.byte_1_hex_right) == u"14" else (u"f" if str(self.byte_1_hex_right) == u"15" else str(self.byte_1_hex_right)))))))
            return getattr(self, '_m_byte_1_hex_right_digit', None)

        @property
        def byte_3_hex_right(self):
            if hasattr(self, '_m_byte_3_hex_right'):
                return self._m_byte_3_hex_right

            self._m_byte_3_hex_right = (self.byte_3 % 16)
            return getattr(self, '_m_byte_3_hex_right', None)

        @property
        def byte_2_hex_right_digit(self):
            if hasattr(self, '_m_byte_2_hex_right_digit'):
                return self._m_byte_2_hex_right_digit

            self._m_byte_2_hex_right_digit = (u"a" if str(self.byte_2_hex_right) == u"10" else (u"b" if str(self.byte_2_hex_right) == u"11" else (u"c" if str(self.byte_2_hex_right) == u"12" else (u"d" if str(self.byte_2_hex_right) == u"13" else (u"e" if str(self.byte_2_hex_right) == u"14" else (u"f" if str(self.byte_2_hex_right) == u"15" else str(self.byte_2_hex_right)))))))
            return getattr(self, '_m_byte_2_hex_right_digit', None)

        @property
        def byte_4_hex_left_digit(self):
            if hasattr(self, '_m_byte_4_hex_left_digit'):
                return self._m_byte_4_hex_left_digit

            self._m_byte_4_hex_left_digit = (u"a" if str(self.byte_4_hex_left) == u"10" else (u"b" if str(self.byte_4_hex_left) == u"11" else (u"c" if str(self.byte_4_hex_left) == u"12" else (u"d" if str(self.byte_4_hex_left) == u"13" else (u"e" if str(self.byte_4_hex_left) == u"14" else (u"f" if str(self.byte_4_hex_left) == u"15" else str(self.byte_4_hex_left)))))))
            return getattr(self, '_m_byte_4_hex_left_digit', None)

        @property
        def byte_4_hex(self):
            if hasattr(self, '_m_byte_4_hex'):
                return self._m_byte_4_hex

            self._m_byte_4_hex = self.byte_4_hex_left_digit + self.byte_4_hex_right_digit
            return getattr(self, '_m_byte_4_hex', None)

        @property
        def batt_v(self):
            if hasattr(self, '_m_batt_v'):
                return self._m_batt_v

            if self.type_id == 0:
                self._m_batt_v = self.byte_1

            return getattr(self, '_m_batt_v', None)

        @property
        def sap_minus_z(self):
            if hasattr(self, '_m_sap_minus_z'):
                return self._m_sap_minus_z

            if self.type_id == 0:
                self._m_sap_minus_z = ((self.byte_5 & 4) >> 2)

            return getattr(self, '_m_sap_minus_z', None)

        @property
        def cw_beacon(self):
            if hasattr(self, '_m_cw_beacon'):
                return self._m_cw_beacon

            self._m_cw_beacon = self.byte_1_hex + u" " + self.byte_2_hex + u" " + self.byte_3_hex + u" " + self.byte_4_hex + u" " + self.byte_5_hex
            return getattr(self, '_m_cw_beacon', None)

        @property
        def byte_2_hex_left_digit(self):
            if hasattr(self, '_m_byte_2_hex_left_digit'):
                return self._m_byte_2_hex_left_digit

            self._m_byte_2_hex_left_digit = (u"a" if str(self.byte_2_hex_left) == u"10" else (u"b" if str(self.byte_2_hex_left) == u"11" else (u"c" if str(self.byte_2_hex_left) == u"12" else (u"d" if str(self.byte_2_hex_left) == u"13" else (u"e" if str(self.byte_2_hex_left) == u"14" else (u"f" if str(self.byte_2_hex_left) == u"15" else str(self.byte_2_hex_left)))))))
            return getattr(self, '_m_byte_2_hex_left_digit', None)

        @property
        def gmsk_cmd_counter(self):
            if hasattr(self, '_m_gmsk_cmd_counter'):
                return self._m_gmsk_cmd_counter

            if self.type_id == 1:
                self._m_gmsk_cmd_counter = ((self.byte_3 & 248) >> 3)

            return getattr(self, '_m_gmsk_cmd_counter', None)

        @property
        def lcl_compic(self):
            if hasattr(self, '_m_lcl_compic'):
                return self._m_lcl_compic

            if self.type_id == 1:
                self._m_lcl_compic = ((self.byte_3 & 2) >> 1)

            return getattr(self, '_m_lcl_compic', None)

        @property
        def lcl_3v3_2(self):
            if hasattr(self, '_m_lcl_3v3_2'):
                return self._m_lcl_3v3_2

            if self.type_id == 1:
                self._m_lcl_3v3_2 = ((self.byte_2 & 32) >> 5)

            return getattr(self, '_m_lcl_3v3_2', None)

        @property
        def raw_i(self):
            if hasattr(self, '_m_raw_i'):
                return self._m_raw_i

            if self.type_id == 1:
                self._m_raw_i = self.byte_1

            return getattr(self, '_m_raw_i', None)

        @property
        def byte_4_hex_left(self):
            if hasattr(self, '_m_byte_4_hex_left'):
                return self._m_byte_4_hex_left

            self._m_byte_4_hex_left = self.byte_4 // 16
            return getattr(self, '_m_byte_4_hex_left', None)

        @property
        def lcl_mainpic(self):
            if hasattr(self, '_m_lcl_mainpic'):
                return self._m_lcl_mainpic

            if self.type_id == 1:
                self._m_lcl_mainpic = ((self.byte_3 & 4) >> 2)

            return getattr(self, '_m_lcl_mainpic', None)

        @property
        def dcdc_fab(self):
            if hasattr(self, '_m_dcdc_fab'):
                return self._m_dcdc_fab

            if self.type_id == 1:
                self._m_dcdc_fab = ((self.byte_2 & 8) >> 3)

            return getattr(self, '_m_dcdc_fab', None)

        @property
        def batt_i(self):
            if hasattr(self, '_m_batt_i'):
                return self._m_batt_i

            if self.type_id == 0:
                self._m_batt_i = self.byte_2

            return getattr(self, '_m_batt_i', None)

        @property
        def kill_counter(self):
            if hasattr(self, '_m_kill_counter'):
                return self._m_kill_counter

            if self.type_id == 1:
                self._m_kill_counter = ((self.byte_4 & 224) >> 5)

            return getattr(self, '_m_kill_counter', None)

        @property
        def kill_sw_main(self):
            if hasattr(self, '_m_kill_sw_main'):
                return self._m_kill_sw_main

            if self.type_id == 0:
                self._m_kill_sw_main = ((self.byte_4 & 16) >> 4)

            return getattr(self, '_m_kill_sw_main', None)

        @property
        def sap_plus_y(self):
            if hasattr(self, '_m_sap_plus_y'):
                return self._m_sap_plus_y

            if self.type_id == 0:
                self._m_sap_plus_y = ((self.byte_4 & 128) >> 7)

            return getattr(self, '_m_sap_plus_y', None)

        @property
        def byte_5_hex_right_digit(self):
            if hasattr(self, '_m_byte_5_hex_right_digit'):
                return self._m_byte_5_hex_right_digit

            self._m_byte_5_hex_right_digit = (u"a" if str(self.byte_5_hex_right) == u"10" else (u"b" if str(self.byte_5_hex_right) == u"11" else (u"c" if str(self.byte_5_hex_right) == u"12" else (u"d" if str(self.byte_5_hex_right) == u"13" else (u"e" if str(self.byte_5_hex_right) == u"14" else (u"f" if str(self.byte_5_hex_right) == u"15" else str(self.byte_5_hex_right)))))))
            return getattr(self, '_m_byte_5_hex_right_digit', None)

        @property
        def byte_3_hex_left_digit(self):
            if hasattr(self, '_m_byte_3_hex_left_digit'):
                return self._m_byte_3_hex_left_digit

            self._m_byte_3_hex_left_digit = (u"a" if str(self.byte_3_hex_left) == u"10" else (u"b" if str(self.byte_3_hex_left) == u"11" else (u"c" if str(self.byte_3_hex_left) == u"12" else (u"d" if str(self.byte_3_hex_left) == u"13" else (u"e" if str(self.byte_3_hex_left) == u"14" else (u"f" if str(self.byte_3_hex_left) == u"15" else str(self.byte_3_hex_left)))))))
            return getattr(self, '_m_byte_3_hex_left_digit', None)

        @property
        def type_id(self):
            if hasattr(self, '_m_type_id'):
                return self._m_type_id

            self._m_type_id = ((self.byte_4 & 128) >> 7)
            return getattr(self, '_m_type_id', None)

        @property
        def dcdc_3v3_2(self):
            if hasattr(self, '_m_dcdc_3v3_2'):
                return self._m_dcdc_3v3_2

            if self.type_id == 1:
                self._m_dcdc_3v3_2 = (self.byte_3 & 1)

            return getattr(self, '_m_dcdc_3v3_2', None)

        @property
        def dcdc_5v0(self):
            if hasattr(self, '_m_dcdc_5v0'):
                return self._m_dcdc_5v0

            if self.type_id == 1:
                self._m_dcdc_5v0 = ((self.byte_2 & 128) >> 7)

            return getattr(self, '_m_dcdc_5v0', None)

        @property
        def byte_2_hex(self):
            if hasattr(self, '_m_byte_2_hex'):
                return self._m_byte_2_hex

            self._m_byte_2_hex = self.byte_2_hex_left_digit + self.byte_2_hex_right_digit
            return getattr(self, '_m_byte_2_hex', None)

        @property
        def byte_4_hex_right(self):
            if hasattr(self, '_m_byte_4_hex_right'):
                return self._m_byte_4_hex_right

            self._m_byte_4_hex_right = (self.byte_4 % 16)
            return getattr(self, '_m_byte_4_hex_right', None)

        @property
        def kill_sw_fab(self):
            if hasattr(self, '_m_kill_sw_fab'):
                return self._m_kill_sw_fab

            if self.type_id == 0:
                self._m_kill_sw_fab = ((self.byte_4 & 32) >> 5)

            return getattr(self, '_m_kill_sw_fab', None)

        @property
        def time_after_reset(self):
            if hasattr(self, '_m_time_after_reset'):
                return self._m_time_after_reset

            if self.type_id == 0:
                self._m_time_after_reset = ((self.byte_5 & 248) >> 3)

            return getattr(self, '_m_time_after_reset', None)

        @property
        def sap_minus_x(self):
            if hasattr(self, '_m_sap_minus_x'):
                return self._m_sap_minus_x

            if self.type_id == 0:
                self._m_sap_minus_x = ((self.byte_4 & 64) >> 6)

            return getattr(self, '_m_sap_minus_x', None)

        @property
        def sap_plus_z(self):
            if hasattr(self, '_m_sap_plus_z'):
                return self._m_sap_plus_z

            if self.type_id == 0:
                self._m_sap_plus_z = ((self.byte_5 & 2) >> 1)

            return getattr(self, '_m_sap_plus_z', None)

        @property
        def uplink_success(self):
            if hasattr(self, '_m_uplink_success'):
                return self._m_uplink_success

            if self.type_id == 1:
                self._m_uplink_success = ((self.byte_4 & 16) >> 4)

            return getattr(self, '_m_uplink_success', None)

        @property
        def byte_5_hex_left(self):
            if hasattr(self, '_m_byte_5_hex_left'):
                return self._m_byte_5_hex_left

            self._m_byte_5_hex_left = self.byte_5 // 16
            return getattr(self, '_m_byte_5_hex_left', None)

        @property
        def byte_1_hex(self):
            if hasattr(self, '_m_byte_1_hex'):
                return self._m_byte_1_hex

            self._m_byte_1_hex = self.byte_1_hex_left_digit + self.byte_1_hex_right_digit
            return getattr(self, '_m_byte_1_hex', None)


    class Hk(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.hk_header_1 = (self._io.read_bytes(14)).decode(u"ASCII")
            if not self.hk_header_1 == u"JG6YBW0JG6YMX0":
                raise kaitaistruct.ValidationNotEqualError(u"JG6YBW0JG6YMX0", self.hk_header_1, self._io, u"/types/hk/seq/0")
            self.hk_header_2 = self._io.read_u8be()
            self.hk_header_3 = self._io.read_u2be()
            if not self.hk_header_3 == 13107:
                raise kaitaistruct.ValidationNotEqualError(13107, self.hk_header_3, self._io, u"/types/hk/seq/2")
            self.hk_seconds = self._io.read_u1()
            self.hk_minutes = self._io.read_u1()
            self.hk_hours = self._io.read_u1()
            self.hk_days = self._io.read_u2be()
            self.hk_delimiter_1 = self._io.read_bits_int_be(24)
            if not self.hk_delimiter_1 == 11184810:
                raise kaitaistruct.ValidationNotEqualError(11184810, self.hk_delimiter_1, self._io, u"/types/hk/seq/7")
            self._io.align_to_byte()
            self.hk_temp_plus_x = self._io.read_u2be()
            self.hk_temp_minus_x = self._io.read_u2be()
            self.hk_temp_plus_y = self._io.read_u2be()
            self.hk_temp_minus_y = self._io.read_u2be()
            self.hk_temp_plus_z = self._io.read_u2be()
            self.hk_temp_minus_z = self._io.read_u2be()
            self.hk_bpb_t = self._io.read_u2be()
            self.hk_voltage_minus_x = self._io.read_u2be()
            self.hk_voltage_plus_y = self._io.read_u2be()
            self.hk_voltage_minus_y = self._io.read_u2be()
            self.hk_voltage_plus_z = self._io.read_u2be()
            self.hk_voltage_minus_z = self._io.read_u2be()
            self.hk_current_minus_x = self._io.read_u1()
            self.hk_current_plus_y = self._io.read_u1()
            self.hk_current_minus_y = self._io.read_u1()
            self.hk_current_plus_z = self._io.read_u1()
            self.hk_current_minus_z = self._io.read_u1()
            self.hk_batt_t = self._io.read_u1()
            self.hk_batt_v = self._io.read_u1()
            self.hk_batt_i = self._io.read_u2be()
            self.hk_fab_raw_v = self._io.read_u1()
            self.hk_fab_raw_i = self._io.read_u1()
            self.hk_src_v = self._io.read_u1()
            self.hk_src_i = self._io.read_u2be()
            self.hk_heater_flag = self._io.read_u1()
            self.hk_kill_sw = self._io.read_u1()
            self.delimiter_2 = self._io.read_u1()
            if not self.delimiter_2 == 187:
                raise kaitaistruct.ValidationNotEqualError(187, self.delimiter_2, self._io, u"/types/hk/seq/34")
            self.hk_reset_raw_v = self._io.read_u1()
            self.hk_v3_3_no1_i = self._io.read_u1()
            self.hk_v3_3_no2_i = self._io.read_u1()
            self.hk_uhf_com_i = self._io.read_u1()
            self.hk_ant_dep_i = self._io.read_u1()
            self.hk_v5_i = self._io.read_u1()
            self.hk_reset_sw_status = self._io.read_u2be()
            self.delimiter_3 = self._io.read_u1()
            if not self.delimiter_3 == 204:
                raise kaitaistruct.ValidationNotEqualError(204, self.delimiter_3, self._io, u"/types/hk/seq/42")
            self.hk_reserve_cmd_0_cmd0 = self._io.read_u1()
            self.hk_reserve_cmd_0_cmd1 = self._io.read_u1()
            self.hk_reserve_cmd_0_cmd2 = self._io.read_u1()
            self.hk_reserve_cmd_0_cmd3 = self._io.read_u1()
            self.hk_reserve_cmd_0_cmd4 = self._io.read_u1()
            self.hk_reserve_cmd_0_cmd5 = self._io.read_u1()
            self.hk_reserve_cmd_0_cmd6 = self._io.read_u1()
            self.hk_reserve_cmd_0_cmd7 = self._io.read_u1()
            self.hk_reserve_cmd_1_cmd0 = self._io.read_u1()
            self.hk_reserve_cmd_1_cmd1 = self._io.read_u1()
            self.hk_reserve_cmd_1_cmd2 = self._io.read_u1()
            self.hk_reserve_cmd_1_cmd3 = self._io.read_u1()
            self.hk_reserve_cmd_1_cmd4 = self._io.read_u1()
            self.hk_reserve_cmd_1_cmd5 = self._io.read_u1()
            self.hk_reserve_cmd_1_cmd6 = self._io.read_u1()
            self.hk_reserve_cmd_1_cmd7 = self._io.read_u1()
            self.delimiter_4 = self._io.read_u4be()
            if not self.delimiter_4 == 1145324612:
                raise kaitaistruct.ValidationNotEqualError(1145324612, self.delimiter_4, self._io, u"/types/hk/seq/59")

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = (u"HK" if 0 == 0 else u"HK")
            return getattr(self, '_m_beacon_type', None)


    class Aprs(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_frame = Yomogi.Aprs.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = Yomogi.Aprs.Ax25Header(self._io, self, self._root)
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Yomogi.Aprs.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dest_callsign_raw = Yomogi.Aprs.CallsignRaw(self._io, self, self._root)
                self.dest_ssid_raw = Yomogi.Aprs.SsidMask(self._io, self, self._root)
                self.src_callsign_raw = Yomogi.Aprs.CallsignRaw(self._io, self, self._root)
                self.src_ssid_raw = Yomogi.Aprs.SsidMask(self._io, self, self._root)
                if (self.src_ssid_raw.ssid_mask & 1) == 0:
                    self.repeater = Yomogi.Aprs.Repeater(self._io, self, self._root)

                self.ctl = self._io.read_u1()
                self.pid = self._io.read_u1()


        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid

                self._m_ssid = ((self.ssid_mask & 31) >> 1)
                return getattr(self, '_m_ssid', None)

            @property
            def hbit(self):
                if hasattr(self, '_m_hbit'):
                    return self._m_hbit

                self._m_hbit = ((self.ssid_mask & 128) >> 7)
                return getattr(self, '_m_hbit', None)


        class Repeaters(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_callsign_raw = Yomogi.Aprs.CallsignRaw(self._io, self, self._root)
                self.rpt_ssid_raw = Yomogi.Aprs.SsidMask(self._io, self, self._root)


        class Repeater(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_instance = []
                i = 0
                while True:
                    _ = Yomogi.Aprs.Repeaters(self._io, self, self._root)
                    self.rpt_instance.append(_)
                    if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                        break
                    i += 1


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Yomogi.Aprs.Callsign(_io__raw_callsign_ror, self, self._root)


        class Ax25InfoData(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.digi_message = (self._io.read_bytes_full()).decode(u"utf-8")


        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = (u"DIGI" if 0 == 0 else u"DIGI")
            return getattr(self, '_m_beacon_type', None)



